@@ -19,19 +19,19 @@ from networkx.generators.degree_seq import *
 z=[5,3,3,3,3,2,2,2,1,1,1]
 is_valid_degree_sequence(z)
 
-print "Configuration model"
+print("Configuration model")
 G=configuration_model(z)  # configuration model
-degree_sequence=degree(G).values() # degree sequence
-print "Degree sequence", degree_sequence
-print "Degree histogram"
+degree_sequence=list(degree(G).values()) # degree sequence
+print("Degree sequence %s" % degree_sequence)
+print("Degree histogram")
 hist={}
 for d in degree_sequence:
-    if hist.has_key(d):
+    if d in hist:
         hist[d]+=1
     else:
         hist[d]=1
-print "degree #nodes"
+print("degree #nodes")
 for d in hist:
-    print d,hist[d]
+    print('%d %d' % (d,hist[d]))
 
 
