@@ -12,7 +12,7 @@ __author__ = """Pieter Swart (swart@lanl.gov)\nDan Schult (dschult@colgate.edu)"
 #    BSD license.
 
 import networkx as nx
-from networkx.exception import NetworkXException, NetworkXError
+from networkx.exception import NetworkXError
 
 __all__ = ['could_be_isomorphic',
            'fast_could_be_isomorphic',
@@ -112,9 +112,9 @@ def faster_could_be_isomorphic(G1,G2):
     if G1.order() != G2.order(): return False
     
     # Check local properties
-    d1=G1.degree().values()
+    d1=list(G1.degree().values())
     d1.sort()
-    d2=G2.degree().values()
+    d2=list(G2.degree().values())
     d2.sort()
 
     if d1 != d2: return False
