@@ -161,7 +161,7 @@ def _bidirectional_pred_succ(G, source, target):
     """
     # does BFS from both source and target and meets in the middle
     if source is None or target is None:
-        raise NetworkXException(\
+        raise nx.NetworkXException(\
             "Bidirectional shortest path called without source or target")
     if target == source:
         return ({target:None},{source:None},source)
