@@ -1,23 +1,18 @@
-"""
-Base class for directed graphs.
-
-"""
-__author__ = """\n""".join(['Aric Hagberg (hagberg@lanl.gov)',
-                            'Pieter Swart (swart@lanl.gov)',
-                            'Dan Schult(dschult@colgate.edu)'])
-#    Copyright (C) 2004-2009 by 
+"""Base class for directed graphs."""
+#    Copyright (C) 2004-2011 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
-#
-
+from copy import deepcopy
 import networkx as nx
 from networkx.classes.graph import Graph
-from networkx.exception import NetworkXException, NetworkXError
+from networkx.exception import NetworkXError
 import networkx.convert as convert
-from copy import deepcopy
+__author__ = """\n""".join(['Aric Hagberg (hagberg@lanl.gov)',
+                            'Pieter Swart (swart@lanl.gov)',
+                            'Dan Schult(dschult@colgate.edu)'])
 
 class DiGraph(Graph):
     """
@@ -42,8 +37,6 @@ class DiGraph(Graph):
         NetworkX graph object.  If the corresponding optional Python
         packages are installed the data can also be a NumPy matrix
         or 2d ndarray, a SciPy sparse matrix, or a PyGraphviz graph.
-    name : string, optional (default='')
-        An optional name for the graph.
     attr : keyword arguments, optional (default= no attributes)
         Attributes to add to graph as key=value pairs.
 
@@ -52,10 +45,10 @@ class DiGraph(Graph):
     Graph
     MultiGraph
     MultiDiGraph
-    
+
     Examples
     --------
-    Create an empty graph structure (a "null graph") with no nodes and 
+    Create an empty graph structure (a "null graph") with no nodes and
     no edges.
 
     >>> G = nx.DiGraph()
@@ -63,17 +56,18 @@ class DiGraph(Graph):
     G can be grown in several ways.
 
     **Nodes:**
-  
+
     Add one node at a time:
 
     >>> G.add_node(1)
 
-    Add the nodes from any container (a list, dict, set or 
+    Add the nodes from any container (a list, dict, set or
     even the lines from a file or the nodes from another graph).
 
     >>> G.add_nodes_from([2,3])
     >>> G.add_nodes_from(range(100,110))
-    >>> H=nx.path_graph(10)
+    >>> H=nx.Graph()
+    >>> H.add_path([0,1,2,3,4,5,6,7,8,9])
     >>> G.add_nodes_from(H)
 
     In addition to strings and integers any hashable Python object
@@ -83,31 +77,31 @@ class DiGraph(Graph):
     >>> G.add_node(H)
 
     **Edges:**
-    
+
     G can also be grown by adding edges.
 
     Add one edge,
 
     >>> G.add_edge(1, 2)
 
-    a list of edges, 
+    a list of edges,
 
     >>> G.add_edges_from([(1,2),(1,3)])
 
     or a collection of edges,
-    
+
     >>> G.add_edges_from(H.edges())
 
-    If some edges connect nodes not yet in the graph, the nodes 
-    are added automatically.  There are no errors when adding 
-    nodes or edges that already exist. 
+    If some edges connect nodes not yet in the graph, the nodes
+    are added automatically.  There are no errors when adding
+    nodes or edges that already exist.
 
     **Attributes:**
-    
+
     Each graph, node, and edge can hold key/value attribute pairs
     in an associated attribute dictionary (the keys must be hashable).
     By default these are empty, but can be added or changed using
-    add_edge, add_node or direct manipulation of the attribute 
+    add_edge, add_node or direct manipulation of the attribute
     dictionaries named graph, node and edge respectively.
 
     >>> G = nx.DiGraph(day="Friday")
@@ -141,36 +135,36 @@ class DiGraph(Graph):
 
     >>> 1 in G     # check if node in graph
     True
-    >>> print [n for n in G if n<3]   # iterate through nodes
+    >>> [n for n in G if n<3]   # iterate through nodes
     [1, 2]
-    >>> print len(G)  # number of nodes in graph
+    >>> len(G)  # number of nodes in graph
     5
-    >>> print G[1] # adjacency dict keyed by neighbor to edge attributes
+    >>> G[1] # adjacency dict keyed by neighbor to edge attributes
     ...            # Note: you should not change this dict manually!
     {2: {'color': 'blue', 'weight': 4}}
 
-    The fastest way to traverse all edges of a graph is via 
+    The fastest way to traverse all edges of a graph is via
     adjacency_iter(), but the edges() method is often more convenient.
 
-    >>> for n,nbrsdict in G.adjacency_iter(): 
-    ...     for nbr,eattr in nbrsdict.iteritems():
+    >>> for n,nbrsdict in G.adjacency_iter():
+    ...     for nbr,eattr in nbrsdict.items():
     ...        if 'weight' in eattr:
-    ...            print (n,nbr,eattr['weight'])
+    ...            (n,nbr,eattr['weight'])
     (1, 2, 4)
     (2, 3, 8)
-    >>> print [ (u,v,edata['weight']) for u,v,edata in G.edges(data=True) if 'weight' in edata ]
+    >>> [ (u,v,edata['weight']) for u,v,edata in G.edges(data=True) if 'weight' in edata ]
     [(1, 2, 4), (2, 3, 8)]
 
     **Reporting:**
-    
+
     Simple graph information is obtained using methods.
     Iterator versions of many reporting methods exist for efficiency.
     Methods exist for reporting nodes(), edges(), neighbors() and degree()
     as well as the number of nodes and edges.
-    
+
     For details on these and other miscellaneous methods, see below.
     """
-    def __init__(self, data=None, name='', **attr):
+    def __init__(self, data=None, **attr):
         """Initialize a graph with edges, name, graph attributes.
 
         Parameters
@@ -189,7 +183,7 @@ class DiGraph(Graph):
         See Also
         --------
         convert
-            
+
         Examples
         --------
         >>> G = nx.Graph()   # or DiGraph, MultiGraph, MultiDiGraph, etc
@@ -212,15 +206,15 @@ class DiGraph(Graph):
         self.adj = {}  # empty adjacency dictionary
         self.pred = {}  # predecessor
         self.succ = self.adj  # successor
-        if data is not None:  # attempt to load graph with data
+
+        # attempt to load graph with data
+        if data is not None:
             convert.to_networkx_graph(data,create_using=self)
         # load graph attributes (must be after convert)
         self.graph.update(attr)
-
-        self.name=name
         self.edge=self.adj
 
-        
+
     def add_node(self, n, attr_dict=None, **attr):
         """Add a single node n and update node attributes.
 
@@ -276,16 +270,17 @@ class DiGraph(Graph):
             self.succ[n] = {}
             self.pred[n] = {}
             self.node[n] = attr_dict
-        else: # update attr even if node already exists            
+        else: # update attr even if node already exists
             self.node[n].update(attr_dict)
 
+
     def add_nodes_from(self, nodes, **attr):
         """Add multiple nodes.
 
         Parameters
         ----------
         nodes : iterable container
-            A container of nodes (list, dict, set, etc.).  
+            A container of nodes (list, dict, set, etc.).
             OR
             A container of (node, attribute dict) tuples.
             Node attributes are updated using the attribute dict.
@@ -304,7 +299,7 @@ class DiGraph(Graph):
         >>> G.add_nodes_from('Hello')
         >>> K3 = nx.Graph([(0,1),(1,2),(2,0)])
         >>> G.add_nodes_from(K3)
-        >>> sorted(G.nodes())
+        >>> sorted(G.nodes(),key=str)
         [0, 1, 2, 'H', 'e', 'l', 'o']
 
         Use keywords to update specific node attributes for every node.
@@ -386,7 +381,7 @@ class DiGraph(Graph):
         for u in nbrs:
             del self.pred[u][n] # remove all edges n-u in digraph
         del self.succ[n]          # remove node from succ
-        for u in self.pred[n]:  
+        for u in self.pred[n]:
             del self.succ[u][n] # remove all edges n-u in digraph
         del self.pred[n]          # remove node from pred
 
@@ -417,25 +412,25 @@ class DiGraph(Graph):
         []
 
         """
-        for n in nbunch: 
+        for n in nbunch:
             try:
                 succs=self.succ[n]
                 del self.node[n]
-                for u in succs:  
+                for u in succs:
                     del self.pred[u][n] # remove all edges n-u in digraph
                 del self.succ[n]          # now remove node
-                for u in self.pred[n]:  
+                for u in self.pred[n]:
                     del self.succ[u][n] # remove all edges n-u in digraph
                 del self.pred[n]          # now remove node
             except KeyError:
                 pass # silent failure on remove
 
 
-    def add_edge(self, u, v, attr_dict=None, **attr):  
+    def add_edge(self, u, v, attr_dict=None, **attr):
         """Add an edge between u and v.
 
-        The nodes u and v will be automatically added if they are 
-        not already in the graph.  
+        The nodes u and v will be automatically added if they are
+        not already in the graph.
 
         Edge attributes can be specified with keywords or by providing
         a dictionary with key/value pairs.  See examples below.
@@ -443,20 +438,20 @@ class DiGraph(Graph):
         Parameters
         ----------
         u,v : nodes
-            Nodes can be, for example, strings or numbers. 
+            Nodes can be, for example, strings or numbers.
             Nodes must be hashable (and not None) Python objects.
         attr_dict : dictionary, optional (default= no attributes)
             Dictionary of edge attributes.  Key/value pairs will
             update existing data associated with the edge.
         attr : keyword arguments, optional
             Edge data (or labels or objects) can be assigned using
-            keyword arguments.   
+            keyword arguments.
 
         See Also
         --------
         add_edges_from : add a collection of edges
 
-        Notes 
+        Notes
         -----
         Adding an edge that already exists updates the edge data.
 
@@ -467,7 +462,7 @@ class DiGraph(Graph):
         Examples
         --------
         The following all add the edge e=(1,2) to graph G:
-        
+
         >>> G = nx.Graph()   # or DiGraph, MultiGraph, MultiDiGraph, etc
         >>> e = (1,2)
         >>> G.add_edge(1, 2)           # explicit two-node form
@@ -488,12 +483,12 @@ class DiGraph(Graph):
             except AttributeError:
                 raise NetworkXError(\
                     "The attr_dict argument must be a dictionary.")
-        # add nodes            
-        if u not in self.succ: 
+        # add nodes
+        if u not in self.succ:
             self.succ[u]={}
             self.pred[u]={}
             self.node[u] = {}
-        if v not in self.succ: 
+        if v not in self.succ:
             self.succ[v]={}
             self.pred[v]={}
             self.node[v] = {}
@@ -503,7 +498,7 @@ class DiGraph(Graph):
         self.succ[u][v]=datadict
         self.pred[v][u]=datadict
 
-    def add_edges_from(self, ebunch, attr_dict=None, **attr):  
+    def add_edges_from(self, ebunch, attr_dict=None, **attr):
         """Add all the edges in ebunch.
 
         Parameters
@@ -518,7 +513,7 @@ class DiGraph(Graph):
             update existing data associated with each edge.
         attr : keyword arguments, optional
             Edge data (or labels or objects) can be assigned using
-            keyword arguments.   
+            keyword arguments.
 
 
         See Also
@@ -537,7 +532,7 @@ class DiGraph(Graph):
         >>> G.add_edges_from([(0,1),(1,2)]) # using a list of edge tuples
         >>> e = zip(range(0,3),range(1,4))
         >>> G.add_edges_from(e) # Add the path graph 0-1-2-3
-  
+
         Associate data to edges
 
         >>> G.add_edges_from([(1,2),(2,3)], weight=3)
@@ -561,19 +556,19 @@ class DiGraph(Graph):
             elif ne==2:
                 u,v = e
                 dd = {}
-            else: 
+            else:
                 raise NetworkXError(\
                     "Edge tuple %s must be a 2-tuple or 3-tuple."%(e,))
-            if u not in self.succ: 
+            if u not in self.succ:
                 self.succ[u] = {}
                 self.pred[u] = {}
                 self.node[u] = {}
-            if v not in self.succ: 
+            if v not in self.succ:
                 self.succ[v] = {}
                 self.pred[v] = {}
                 self.node[v] = {}
             datadict=self.adj[u].get(v,{})
-            datadict.update(attr_dict) 
+            datadict.update(attr_dict)
             datadict.update(dd)
             self.succ[u][v] = datadict
             self.pred[v][u] = datadict
@@ -584,7 +579,7 @@ class DiGraph(Graph):
 
         Parameters
         ----------
-        u,v: nodes 
+        u,v: nodes
             Remove the edge between nodes u and v.
 
         Raises
@@ -607,19 +602,19 @@ class DiGraph(Graph):
         >>> G.remove_edge(*e[:2]) # select first part of edge tuple
         """
         try:
-            del self.succ[u][v]   
-            del self.pred[v][u]   
-        except KeyError: 
+            del self.succ[u][v]
+            del self.pred[v][u]
+        except KeyError:
             raise NetworkXError("The edge %s-%s not in graph."%(u,v))
 
 
-    def remove_edges_from(self, ebunch): 
+    def remove_edges_from(self, ebunch):
         """Remove all edges specified in ebunch.
 
         Parameters
         ----------
         ebunch: list or container of edge tuples
-            Each edge given in the list or container will be removed 
+            Each edge given in the list or container will be removed
             from the graph. The edges can be:
 
                 - 2-tuples (u,v) edge between u and v.
@@ -628,7 +623,7 @@ class DiGraph(Graph):
         See Also
         --------
         remove_edge : remove a single edge
-            
+
         Notes
         -----
         Will fail silently if an edge in ebunch is not in the graph.
@@ -638,13 +633,13 @@ class DiGraph(Graph):
         >>> G = nx.Graph()   # or DiGraph, MultiGraph, MultiDiGraph, etc
         >>> G.add_path([0,1,2,3])
         >>> ebunch=[(1,2),(2,3)]
-        >>> G.remove_edges_from(ebunch) 
+        >>> G.remove_edges_from(ebunch)
         """
         for e in ebunch:
             (u,v)=e[:2]  # ignore edge data
             if u in self.succ and v in self.succ[u]:
-                del self.succ[u][v]   
-                del self.pred[v][u]        
+                del self.succ[u][v]
+                del self.pred[v][u]
 
 
     def has_successor(self, u, v):
@@ -659,28 +654,28 @@ class DiGraph(Graph):
 
         This is true if graph has the edge u<-v.
         """
-        return (u in self.pred and v in self.pred[u])    
+        return (u in self.pred and v in self.pred[u])
 
     def successors_iter(self,n):
         """Return an iterator over successor nodes of n.
-        
+
         neighbors_iter() and successors_iter() are the same.
         """
         try:
-            return self.succ[n].iterkeys()
+            return iter(self.succ[n].keys())
         except KeyError:
             raise NetworkXError("The node %s is not in the digraph."%(n,))
 
     def predecessors_iter(self,n):
         """Return an iterator over predecessor nodes of n."""
         try:
-            return self.pred[n].iterkeys()
+            return iter(self.pred[n].keys())
         except KeyError:
             raise NetworkXError("The node %s is not in the digraph."%(n,))
 
     def successors(self, n):
         """Return a list of successor nodes of n.
-        
+
         neighbors() and successors() are the same function.
         """
         return list(self.successors_iter(n))
@@ -690,14 +685,14 @@ class DiGraph(Graph):
         return list(self.predecessors_iter(n))
 
 
-    # digraph definitions 
+    # digraph definitions
     neighbors = successors
     neighbors_iter = successors_iter
 
     def edges_iter(self, nbunch=None, data=False):
         """Return an iterator over the edges.
-        
-        Edges are returned as tuples with optional data 
+
+        Edges are returned as tuples with optional data
         in the order (node, neighbor, data).
 
         Parameters
@@ -736,12 +731,12 @@ class DiGraph(Graph):
 
         """
         if nbunch is None:
-            nodes_nbrs=self.adj.iteritems()
+            nodes_nbrs=iter(self.adj.items())
         else:
             nodes_nbrs=((n,self.adj[n]) for n in self.nbunch_iter(nbunch))
         if data:
             for n,nbrs in nodes_nbrs:
-                for nbr,data in nbrs.iteritems():
+                for nbr,data in nbrs.items():
                     yield (n,nbr,data)
         else:
             for n,nbrs in nodes_nbrs:
@@ -754,7 +749,7 @@ class DiGraph(Graph):
 
     def in_edges_iter(self, nbunch=None, data=False):
         """Return an iterator over the incoming edges.
-        
+
         Parameters
         ----------
         nbunch : iterable container, optional (default= all nodes)
@@ -773,12 +768,12 @@ class DiGraph(Graph):
         edges_iter : return an iterator of edges
         """
         if nbunch is None:
-            nodes_nbrs=self.pred.iteritems()
+            nodes_nbrs=iter(self.pred.items())
         else:
             nodes_nbrs=((n,self.pred[n]) for n in self.nbunch_iter(nbunch))
         if data:
             for n,nbrs in nodes_nbrs:
-                for nbr,data in nbrs.iteritems():
+                for nbr,data in nbrs.items():
                     yield (nbr,n,data)
         else:
             for n,nbrs in nodes_nbrs:
@@ -795,23 +790,23 @@ class DiGraph(Graph):
         return list(self.in_edges_iter(nbunch, data))
 
     def degree_iter(self, nbunch=None, weighted=False):
-        """Return an iterator for (node, degree). 
+        """Return an iterator for (node, degree).
 
-        The node degree is the number of edges adjacent to the node. 
+        The node degree is the number of edges adjacent to the node.
 
         Parameters
         ----------
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
-            through once.    
+            through once.
         weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.  
+           If True return the sum of edge weights adjacent to the node.
 
         Returns
         -------
-        nd_iter : an iterator 
+        nd_iter : an iterator
             The iterator returns two-tuples of (node, degree).
-        
+
         See Also
         --------
         degree, in_degree, out_degree, in_degree_iter, out_degree_iter
@@ -826,11 +821,10 @@ class DiGraph(Graph):
         [(0, 1), (1, 2)]
 
         """
-        from itertools import izip
         if nbunch is None:
-            nodes_nbrs=izip(self.succ.iteritems(),self.pred.iteritems())
+            nodes_nbrs=zip(iter(self.succ.items()),iter(self.pred.items()))
         else:
-            nodes_nbrs=izip(
+            nodes_nbrs=zip(
                 ((n,self.succ[n]) for n in self.nbunch_iter(nbunch)),
                 ((n,self.pred[n]) for n in self.nbunch_iter(nbunch)))
 
@@ -844,8 +838,9 @@ class DiGraph(Graph):
             for (n,succ),(n2,pred) in nodes_nbrs:
                 yield (n,len(succ)+len(pred)) 
 
+
     def in_degree_iter(self, nbunch=None, weighted=False):
-        """Return an iterator for (node, in-degree). 
+        """Return an iterator for (node, in-degree).
 
         The node in-degree is the number of edges pointing in to the node.
 
@@ -853,22 +848,22 @@ class DiGraph(Graph):
         ----------
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
-            through once.    
+            through once.
         weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.  
+           If True return the sum of edge weights adjacent to the node.
 
         Returns
         -------
-        nd_iter : an iterator 
+        nd_iter : an iterator
             The iterator returns two-tuples of (node, in-degree).
-        
+
         See Also
         --------
         degree, in_degree, out_degree, out_degree_iter
 
         Examples
         --------
-        >>> G = nx.DiGraph()   
+        >>> G = nx.DiGraph()
         >>> G.add_path([0,1,2,3])
         >>> list(G.in_degree_iter(0)) # node 0 with degree 0
         [(0, 0)]
@@ -877,21 +872,21 @@ class DiGraph(Graph):
 
         """
         if nbunch is None:
-            nodes_nbrs=self.pred.iteritems()
+            nodes_nbrs=iter(self.pred.items())
         else:
             nodes_nbrs=((n,self.pred[n]) for n in self.nbunch_iter(nbunch))
-  
-        if weighted:                        
+
+        if weighted:
         # edge weighted graph - degree is sum of edge weights
             for n,nbrs in nodes_nbrs:
-                yield (n, sum((nbrs[nbr].get('weight',1) for nbr in nbrs)))
+                yield (n, sum(data.get('weight',1) for data in nbrs.values()))
         else:
             for n,nbrs in nodes_nbrs:
                 yield (n,len(nbrs))
 
 
     def out_degree_iter(self, nbunch=None, weighted=False):
-        """Return an iterator for (node, out-degree). 
+        """Return an iterator for (node, out-degree).
 
         The node out-degree is the number of edges pointing out of the node.
 
@@ -899,15 +894,15 @@ class DiGraph(Graph):
         ----------
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
-            through once.    
+            through once.
         weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.  
+           If True return the sum of edge weights adjacent to the node.
 
         Returns
         -------
-        nd_iter : an iterator 
+        nd_iter : an iterator
             The iterator returns two-tuples of (node, out-degree).
-        
+
         See Also
         --------
         degree, in_degree, out_degree, in_degree_iter
@@ -923,14 +918,14 @@ class DiGraph(Graph):
 
         """
         if nbunch is None:
-            nodes_nbrs=self.succ.iteritems()
+            nodes_nbrs=iter(self.succ.items())
         else:
             nodes_nbrs=((n,self.succ[n]) for n in self.nbunch_iter(nbunch))
-  
-        if weighted:                        
+
+        if weighted:
         # edge weighted graph - degree is sum of edge weights
             for n,nbrs in nodes_nbrs:
-                yield (n, sum((nbrs[nbr].get('weight',1) for nbr in nbrs)))
+                yield (n, sum(data.get('weight',1) for data in nbrs.values()))
         else:
             for n,nbrs in nodes_nbrs:
                 yield (n,len(nbrs))
@@ -945,9 +940,9 @@ class DiGraph(Graph):
         ----------
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
-            through once.    
+            through once.
         weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.  
+           If True return the sum of edge weights adjacent to the node.
 
         Returns
         -------
@@ -967,11 +962,11 @@ class DiGraph(Graph):
         0
         >>> G.in_degree([0,1])
         {0: 0, 1: 1}
-        >>> G.in_degree([0,1]).values()
+        >>> list(G.in_degree([0,1]).values())
         [0, 1]
         """
         if nbunch in self:      # return a single node
-            return self.in_degree_iter(nbunch,weighted=weighted).next()[1]
+            return next(self.in_degree_iter(nbunch,weighted=weighted))[1]
         else:           # return a dict
             return dict(self.in_degree_iter(nbunch,weighted=weighted))
 
@@ -984,9 +979,9 @@ class DiGraph(Graph):
         ----------
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
-            through once.    
+            through once.
         weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.  
+           If True return the sum of edge weights adjacent to the node.
 
         Returns
         -------
@@ -1002,13 +997,13 @@ class DiGraph(Graph):
         1
         >>> G.out_degree([0,1])
         {0: 1, 1: 1}
-        >>> G.out_degree([0,1]).values()
+        >>> list(G.out_degree([0,1]).values())
         [1, 1]
 
 
         """
         if nbunch in self:      # return a single node
-            return self.out_degree_iter(nbunch,weighted=weighted).next()[1]
+            return next(self.out_degree_iter(nbunch,weighted=weighted))[1]
         else:           # return a dict
             return dict(self.out_degree_iter(nbunch,weighted=weighted))
 
@@ -1016,7 +1011,7 @@ class DiGraph(Graph):
         """Remove all nodes and edges from the graph.
 
         This also removes the name, and all graph, node, and edge attributes.
-        
+
         Examples
         --------
         >>> G = nx.Graph()   # or DiGraph, MultiGraph, MultiDiGraph, etc
@@ -1028,9 +1023,8 @@ class DiGraph(Graph):
         []
 
         """
-        self.name=''
-        self.succ.clear() 
-        self.pred.clear() 
+        self.succ.clear()
+        self.pred.clear()
         self.node.clear()
         self.graph.clear()
 
@@ -1046,7 +1040,7 @@ class DiGraph(Graph):
 
     def to_directed(self):
         """Return a directed copy of the graph.
- 
+
         Returns
         -------
         G : DiGraph
@@ -1054,12 +1048,12 @@ class DiGraph(Graph):
 
         Notes
         -----
-        This returns a "deepcopy" of the edge, node, and 
+        This returns a "deepcopy" of the edge, node, and
         graph attributes which attempts to completely copy
         all of the data and references.
 
-        This is in contrast to the similar D=DiGraph(G) which returns a 
-        shallow copy of the data. 
+        This is in contrast to the similar D=DiGraph(G) which returns a
+        shallow copy of the data.
 
         See the Python copy module for more information on shallow
         and deep copies, http://docs.python.org/library/copy.html.
@@ -1082,13 +1076,19 @@ class DiGraph(Graph):
         """
         return deepcopy(self)
 
-    def to_undirected(self):
+    def to_undirected(self, reciprocal=False):
         """Return an undirected representation of the digraph.
- 
+
+        Parameters
+        ----------
+        reciprocal : bool (optional)
+          If True only keep edges that appear in both directions 
+          in the original digraph. 
+
         Returns
         -------
         G : Graph
-            An undirected graph with the same name and nodes and 
+            An undirected graph with the same name and nodes and
             with edge (u,v,data) if either (u,v,data) or (v,u,data)
             is in the digraph.  If both edges exist in digraph and
             their edge data is different, only one edge is created
@@ -1103,31 +1103,36 @@ class DiGraph(Graph):
         in the (arbitrary) order that the edges are encountered.  For
         more customized control of the edge attributes use add_edge().
 
-        This returns a "deepcopy" of the edge, node, and 
+        This returns a "deepcopy" of the edge, node, and
         graph attributes which attempts to completely copy
         all of the data and references.
 
-        This is in contrast to the similar G=DiGraph(D) which returns a 
-        shallow copy of the data. 
+        This is in contrast to the similar G=DiGraph(D) which returns a
+        shallow copy of the data.
 
         See the Python copy module for more information on shallow
         and deep copies, http://docs.python.org/library/copy.html.
-
         """
         H=Graph()
         H.name=self.name
         H.add_nodes_from(self)
-        H.add_edges_from( (u,v,deepcopy(d)) 
-                          for u,nbrs in self.adjacency_iter()
-                          for v,d in nbrs.iteritems() )
+        if reciprocal is True:
+            H.add_edges_from( (u,v,deepcopy(d))
+                              for u,nbrs in self.adjacency_iter()
+                              for v,d in nbrs.items() 
+                              if v in self.pred[u])
+        else:
+            H.add_edges_from( (u,v,deepcopy(d))
+                              for u,nbrs in self.adjacency_iter()
+                              for v,d in nbrs.items() )
         H.graph=deepcopy(self.graph)
         H.node=deepcopy(self.node)
         return H
-    
+
 
     def reverse(self, copy=True):
         """Return the reverse of the graph.
-        
+
         The reverse is a graph with the same nodes and edges
         but with the directions of the edges reversed.
 
@@ -1155,18 +1160,18 @@ class DiGraph(Graph):
     def subgraph(self, nbunch):
         """Return the subgraph induced on nodes in nbunch.
 
-        The induced subgraph of the graph contains the nodes in nbunch 
-        and the edges between those nodes.  
-        
+        The induced subgraph of the graph contains the nodes in nbunch
+        and the edges between those nodes.
+
         Parameters
         ----------
         nbunch : list, iterable
-            A container of nodes which will be iterated through once.    
+            A container of nodes which will be iterated through once.
 
         Returns
         -------
         G : Graph
-            A subgraph of the graph with the same edge attributes.  
+            A subgraph of the graph with the same edge attributes.
 
         Notes
         -----
@@ -1180,26 +1185,24 @@ class DiGraph(Graph):
         If edge attributes are containers, a deep copy can be obtained using:
         G.subgraph(nbunch).copy()
 
-        For an in-place reduction of a graph to a subgraph you can remove nodes:
-        G.remove_nodes_from([ n in G if n not in set(nbunch)])  
+        For an inplace reduction of a graph to a subgraph you can remove nodes:
+        G.remove_nodes_from([ n in G if n not in set(nbunch)])
 
         Examples
         --------
         >>> G = nx.Graph()   # or DiGraph, MultiGraph, MultiDiGraph, etc
         >>> G.add_path([0,1,2,3])
         >>> H = G.subgraph([0,1,2])
-        >>> print H.edges()
+        >>> H.edges()
         [(0, 1), (1, 2)]
         """
         bunch = self.nbunch_iter(nbunch)
-        # create new graph and copy subgraph into it       
+        # create new graph and copy subgraph into it
         H = self.__class__()
-        H.name = "Subgraph of (%s)"%(self.name)
         # namespace shortcuts for speed
         H_succ=H.succ
         H_pred=H.pred
         self_succ=self.succ
-        self_pred=self.pred
         # add nodes
         for n in bunch:
             H_succ[n]={}
@@ -1207,7 +1210,7 @@ class DiGraph(Graph):
         # add edges
         for u in H_succ:
             Hnbrs=H_succ[u]
-            for v,datadict in self_succ[u].iteritems():
+            for v,datadict in self_succ[u].items():
                 if v in H_succ:
                     # add both representations of edge: u-v and v-u
                     Hnbrs[v]=datadict
