@@ -1,26 +1,28 @@
 """
 Functional interface to graph methods and assorted utilities.
-    
+
 """
 __author__ = """\n""".join(['Aric Hagberg (hagberg@lanl.gov)',
                            'Pieter Swart (swart@lanl.gov)',
                            'Dan Schult(dschult@colgate.edu)'])
-#    Copyright (C) 2004-2010 by 
+#    Copyright (C) 2004-2010 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
 #
-
 import networkx as nx
+
 # functional style helpers
 
 
 __all__ = ['nodes', 'edges', 'degree', 'degree_histogram', 'neighbors',
            'number_of_nodes', 'number_of_edges', 'density',
            'nodes_iter', 'edges_iter', 'is_directed','info',
-           'freeze','is_frozen','subgraph','create_empty_copy']
+           'freeze','is_frozen','subgraph','create_empty_copy',
+           'set_node_attributes','get_node_attributes',
+           'set_edge_attributes','get_edge_attributes']
 
 def nodes(G):
     """Return a copy of the graph nodes in a list."""
@@ -65,24 +67,23 @@ def number_of_nodes(G):
 def number_of_edges(G):
     """Return the number of edges in the graph. """
     return G.number_of_edges()
-    
+
 def density(G):
-    """Return the density of a graph.
+    r"""Return the density of a graph.
 
     The density for undirected graphs is
-    
-    .. math::
-
-       d = \\frac{2m}{n(n-1)},
-       
-    and for directed graphs is        
 
     .. math::
 
-       d = \\frac{m}{n(n-1)},
+       d = \frac{2m}{n(n-1)},
 
-    where :math:`n` is the number of nodes and :math:`m` 
-    is the number of edges in :math:`G`.
+    and for directed graphs is
+
+    .. math::
+
+       d = \frac{m}{n(n-1)},
+
+    where `n` is the number of nodes and `m`  is the number of edges in `G`.
 
     Notes
     -----
@@ -104,7 +105,7 @@ def density(G):
 
 def degree_histogram(G):
     """Return a list of the frequency of each degree value.
-    
+
     Parameters
     ----------
     G : Networkx graph
@@ -115,15 +116,15 @@ def degree_histogram(G):
     hist : list
        A list of frequencies of degrees.
        The degree values are the index in the list.
-    
+
     Notes
     -----
     Note: the bins are width one, hence len(list) can be large
     (Order(number_of_edges))
     """
-    degseq=G.degree().values()
+    degseq=list(G.degree().values())
     dmax=max(degseq)+1
-    freq= [ 0 for d in xrange(dmax) ]
+    freq= [ 0 for d in range(dmax) ]
     for d in degseq:
         freq[d] += 1
     return freq
@@ -132,44 +133,10 @@ def is_directed(G):
     """ Return True if graph is directed."""
     return G.is_directed()
 
-def info(G, n=None):
-    """Print short summary of information for graph G or node n."""
-    import textwrap
-    width_left = 22
-
-    if n is None:
-        print ("Name:").ljust(width_left), G.name
-        type_name = [type(G).__name__]
-        print ("Type:").ljust(width_left), ",".join(type_name)
-        print ("Number of nodes:").ljust(width_left), G.number_of_nodes()
-        print ("Number of edges:").ljust(width_left), G.number_of_edges()
-        if len(G) > 0:
-            if G.is_directed():
-                print ("Average in degree:").ljust(width_left), \
-                    round( sum(G.in_degree())/float(len(G)), 4)
-                print ("Average out degree:").ljust(width_left), \
-                    round( sum(G.out_degree())/float(len(G)), 4)
-            else:
-                print ("Average degree:").ljust(width_left), \
-                    round( sum(G.degree())/float(len(G)), 4)
-
-    else:
-        try:
-            list_neighbors = G.neighbors(n)
-        except (KeyError, TypeError):
-            raise NetworkXError, "node %s not in graph"%(n,)
-        print "\nNode", n, "has the following properties:"
-        print ("Degree:").ljust(width_left), len(list_neighbors)
-        str_neighbors = str(list_neighbors)
-        str_neighbors = str_neighbors[1:len(str_neighbors)-1]
-        wrapped_neighbors = textwrap.wrap(str_neighbors, 50)
-        print ("Neighbors:").ljust(width_left), wrapped_neighbors[0]
-        for i in wrapped_neighbors[1:]:
-            print "".ljust(width_left), i
 
 def freeze(G):
     """Modify graph to prevent addition of nodes or edges.
-    
+
     Parameters
     -----------
     G : graph
@@ -177,12 +144,14 @@ def freeze(G):
 
     Examples
     --------
-    >>> G=nx.path_graph(4)
+    >>> G=nx.Graph()
+    >>> G.add_path([0,1,2,3])
     >>> G=nx.freeze(G)
-    >>> G.add_edge(4,5)
-    Traceback (most recent call last):
-    ...
-    NetworkXError: Frozen graph can't be modified
+    >>> try:
+    ...    G.add_edge(4,5)
+    ... except nx.NetworkXError as e:
+    ...    print(str(e))
+    Frozen graph can't be modified
 
     Notes
     -----
@@ -194,8 +163,8 @@ def freeze(G):
     --------
     is_frozen
 
-    """        
-    def frozen(*args):    
+    """
+    def frozen(*args):
         raise nx.NetworkXError("Frozen graph can't be modified")
     G.add_node=frozen
     G.add_nodes_from=frozen
@@ -232,9 +201,9 @@ def subgraph(G, nbunch):
     Parameters
     ----------
     G : graph
-       A NetworkX graph 
+       A NetworkX graph
 
-    nbunch : list, iterable 
+    nbunch : list, iterable
        A container of nodes that will be iterated through once (thus
        it should be an iterator or be iterable).  Each element of the
        container should be a valid node type: any hashable type except
@@ -255,19 +224,152 @@ def create_empty_copy(G,with_nodes=True):
     Parameters
     ----------
     G : graph
-       A NetworkX graph 
+       A NetworkX graph
 
     with_nodes :  bool (default=True)
-       Include nodes. 
+       Include nodes.
 
     Notes
     -----
     Graph, node, and edge data is not propagated to the new graph.
     """
     H=G.__class__()
-
-    H.name='empty '+G.name
     if with_nodes:
         H.add_nodes_from(G)
     return H
 
+
+def info(G, n=None):
+    """Print short summary of information for the graph G or the node n.
+
+    Parameters
+    ----------
+    G : Networkx graph
+       A graph
+    n : node (any hashable)
+       A node in the graph G
+    """
+    info='' # append this all to a string
+    if n is None:
+        info+="Name: %s\n"%G.name
+        type_name = [type(G).__name__]
+        info+="Type: %s\n"%",".join(type_name)
+        info+="Number of nodes: %d\n"%G.number_of_nodes()
+        info+="Number of edges: %d\n"%G.number_of_edges()
+        nnodes=G.number_of_nodes()
+        if len(G) > 0:
+            if G.is_directed():
+                info+="Average in degree: %8.4f\n"%\
+                    (sum(G.in_degree().values())/float(nnodes))
+                info+="Average out degree: %8.4f"%\
+                    (sum(G.out_degree().values())/float(nnodes))
+            else:
+                s=sum(G.degree().values())
+                info+="Average degree: %8.4f"%\
+                    (float(s)/float(nnodes))
+
+    else:
+        if n not in G:
+            raise nx.NetworkXError("node %s not in graph"%(n,))
+        info+="Node % s has the following properties:\n"%n
+        info+="Degree: %d\n"%G.degree(n)
+        info+="Neighbors: "
+        info+=' '.join(str(nbr) for nbr in G.neighbors(n))
+    return info
+
+def set_node_attributes(G,name,attributes):
+    """Set node attributes from dictionary of nodes and values
+
+    Parameters
+    ----------
+    G : NetworkX Graph
+    
+    name : string
+       Attribute name
+
+    attributes: dict
+       Dictionary of attributes keyed by node.
+
+    Examples
+    --------
+    >>> G=nx.path_graph(3)
+    >>> bb=nx.betweenness_centrality(G)
+    >>> nx.set_node_attributes(G,'betweenness',bb)
+    >>> G.node[1]['betweenness']
+    1.0
+    """
+    for node,value in attributes.items():
+        G.node[node][name]=value
+
+def get_node_attributes(G,name):
+    """Get node attributes from graph
+
+    Parameters
+    ----------
+    G : NetworkX Graph
+    
+    name : string
+       Attribute name
+
+    Returns
+    -------
+    Dictionary of attributes keyed by node.
+
+    Examples
+    --------
+    >>> G=nx.Graph()
+    >>> G.add_nodes_from([1,2,3],color='red')
+    >>> color=nx.get_node_attributes(G,'color')
+    >>> color[1]
+    'red'
+    """
+    return dict( (n,d[name]) for n,d in G.node.items() if name in d)
+
+
+def set_edge_attributes(G,name,attributes):
+    """Set edge attributes from dictionary of edge tuples and values
+
+    Parameters
+    ----------
+    G : NetworkX Graph
+    
+    name : string
+       Attribute name
+
+    attributes: dict
+       Dictionary of attributes keyed by edge (tuple).
+
+    Examples
+    --------
+    >>> G=nx.path_graph(3)
+    >>> bb=nx.edge_betweenness_centrality(G)
+    >>> nx.set_edge_attributes(G,'betweenness',bb)
+    >>> G[1][2]['betweenness']
+    4.0
+    """
+    for (u,v),value in attributes.items():
+        G[u][v][name]=value
+
+def get_edge_attributes(G,name):
+    """Get edge attributes from graph
+
+    Parameters
+    ----------
+    G : NetworkX Graph
+    
+    name : string
+       Attribute name
+
+    Returns
+    -------
+    Dictionary of attributes keyed by node.
+
+    Examples
+    --------
+    >>> G=nx.Graph()
+    >>> G.add_path([1,2,3],color='red')
+    >>> color=nx.get_edge_attributes(G,'color')
+    >>> color[(1,2)]
+    'red'
+    """
+    return dict( ((u,v),d[name]) for u,v,d in G.edges(data=True) if name in d)
