@@ -1,31 +1,23 @@
-"""
-Base class for MultiDiGraph.
-
-"""
-__author__ = """\n""".join(['Aric Hagberg (hagberg@lanl.gov)',
-                            'Pieter Swart (swart@lanl.gov)',
-                            'Dan Schult(dschult@colgate.edu)'])
-
-#    Copyright (C) 2004-2009 by
+"""Base class for MultiDiGraph."""
+#    Copyright (C) 2004-2011 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
-#
-
+from copy import deepcopy
 import networkx as nx
 from networkx.classes.graph import Graph  # for doctests
 from networkx.classes.digraph import DiGraph
 from networkx.classes.multigraph import MultiGraph
-from networkx.exception import NetworkXException, NetworkXError
-from copy import deepcopy
-
+from networkx.exception import NetworkXError
+__author__ = """\n""".join(['Aric Hagberg (hagberg@lanl.gov)',
+                            'Pieter Swart (swart@lanl.gov)',
+                            'Dan Schult(dschult@colgate.edu)'])
 
 class MultiDiGraph(MultiGraph,DiGraph):
-    """
-    A directed graph class that can store multiedges.
-    
+    """A directed graph class that can store multiedges.
+
     Multiedges are multiple edges between two nodes.  Each edge
     can hold optional data or attributes.
 
@@ -45,8 +37,6 @@ class MultiDiGraph(MultiGraph,DiGraph):
         NetworkX graph object.  If the corresponding optional Python
         packages are installed the data can also be a NumPy matrix
         or 2d ndarray, a SciPy sparse matrix, or a PyGraphviz graph.
-    name : string, optional (default='')
-        An optional name for the graph.
     attr : keyword arguments, optional (default= no attributes)
         Attributes to add to graph as key=value pairs.
 
@@ -55,10 +45,10 @@ class MultiDiGraph(MultiGraph,DiGraph):
     Graph
     DiGraph
     MultiGraph
-    
+
     Examples
     --------
-    Create an empty graph structure (a "null graph") with no nodes and 
+    Create an empty graph structure (a "null graph") with no nodes and
     no edges.
 
     >>> G = nx.MultiDiGraph()
@@ -66,17 +56,18 @@ class MultiDiGraph(MultiGraph,DiGraph):
     G can be grown in several ways.
 
     **Nodes:**
-  
+
     Add one node at a time:
 
     >>> G.add_node(1)
 
-    Add the nodes from any container (a list, dict, set or 
+    Add the nodes from any container (a list, dict, set or
     even the lines from a file or the nodes from another graph).
 
     >>> G.add_nodes_from([2,3])
     >>> G.add_nodes_from(range(100,110))
-    >>> H=nx.path_graph(10)
+    >>> H=nx.Graph()
+    >>> H.add_path([0,1,2,3,4,5,6,7,8,9])
     >>> G.add_nodes_from(H)
 
     In addition to strings and integers any hashable Python object
@@ -86,22 +77,22 @@ class MultiDiGraph(MultiGraph,DiGraph):
     >>> G.add_node(H)
 
     **Edges:**
-    
+
     G can also be grown by adding edges.
 
     Add one edge,
 
     >>> G.add_edge(1, 2)
 
-    a list of edges, 
+    a list of edges,
 
     >>> G.add_edges_from([(1,2),(1,3)])
 
     or a collection of edges,
-    
+
     >>> G.add_edges_from(H.edges())
 
-    If some edges connect nodes not yet in the graph, the nodes 
+    If some edges connect nodes not yet in the graph, the nodes
     are added automatically.  If an edge already exists, an additional
     edge is created and stored using a key to identify the edge.
     By default the key is the lowest unused integer.
@@ -111,11 +102,11 @@ class MultiDiGraph(MultiGraph,DiGraph):
     {5: {0: {}, 1: {'route': 282}, 2: {'route': 37}}}
 
     **Attributes:**
-    
+
     Each graph, node, and edge can hold key/value attribute pairs
     in an associated attribute dictionary (the keys must be hashable).
     By default these are empty, but can be added or changed using
-    add_edge, add_node or direct manipulation of the attribute 
+    add_edge, add_node or direct manipulation of the attribute
     dictionaries named graph, node and edge respectively.
 
     >>> G = nx.MultiDiGraph(day="Friday")
@@ -149,41 +140,41 @@ class MultiDiGraph(MultiGraph,DiGraph):
 
     >>> 1 in G     # check if node in graph
     True
-    >>> print [n for n in G if n<3]   # iterate through nodes
+    >>> [n for n in G if n<3]   # iterate through nodes
     [1, 2]
-    >>> print len(G)  # number of nodes in graph
+    >>> len(G)  # number of nodes in graph
     5
-    >>> print G[1] # adjacency dict keyed by neighbor to edge attributes
+    >>> G[1] # adjacency dict keyed by neighbor to edge attributes
     ...            # Note: you should not change this dict manually!
     {2: {0: {'weight': 4}, 1: {'color': 'blue'}}}
 
-    The fastest way to traverse all edges of a graph is via 
+    The fastest way to traverse all edges of a graph is via
     adjacency_iter(), but the edges() method is often more convenient.
 
-    >>> for n,nbrsdict in G.adjacency_iter(): 
-    ...     for nbr,keydict in nbrsdict.iteritems():
-    ...        for key,eattr in keydict.iteritems():
+    >>> for n,nbrsdict in G.adjacency_iter():
+    ...     for nbr,keydict in nbrsdict.items():
+    ...        for key,eattr in keydict.items():
     ...            if 'weight' in eattr:
-    ...                print (n,nbr,eattr['weight'])
+    ...                (n,nbr,eattr['weight'])
     (1, 2, 4)
     (2, 3, 8)
-    >>> print [ (u,v,edata['weight']) for u,v,edata in G.edges(data=True) if 'weight' in edata ]
+    >>> [ (u,v,edata['weight']) for u,v,edata in G.edges(data=True) if 'weight' in edata ]
     [(1, 2, 4), (2, 3, 8)]
 
     **Reporting:**
-    
+
     Simple graph information is obtained using methods.
     Iterator versions of many reporting methods exist for efficiency.
     Methods exist for reporting nodes(), edges(), neighbors() and degree()
     as well as the number of nodes and edges.
-    
+
     For details on these and other miscellaneous methods, see below.
     """
     def add_edge(self, u, v, key=None, attr_dict=None, **attr):
         """Add an edge between u and v.
 
-        The nodes u and v will be automatically added if they are 
-        not already in the graph.  
+        The nodes u and v will be automatically added if they are
+        not already in the graph.
 
         Edge attributes can be specified with keywords or by providing
         a dictionary with key/value pairs.  See examples below.
@@ -191,22 +182,22 @@ class MultiDiGraph(MultiGraph,DiGraph):
         Parameters
         ----------
         u,v : nodes
-            Nodes can be, for example, strings or numbers. 
+            Nodes can be, for example, strings or numbers.
             Nodes must be hashable (and not None) Python objects.
         key : hashable identifier, optional (default=lowest unused integer)
-            Used to distinguish multiedges between a pair of nodes.  
+            Used to distinguish multiedges between a pair of nodes.
         attr_dict : dictionary, optional (default= no attributes)
             Dictionary of edge attributes.  Key/value pairs will
             update existing data associated with the edge.
         attr : keyword arguments, optional
             Edge data (or labels or objects) can be assigned using
-            keyword arguments.   
+            keyword arguments.
 
         See Also
         --------
         add_edges_from : add a collection of edges
 
-        Notes 
+        Notes
         -----
         To replace/update edge data, use the optional key argument
         to identify a unique edge.  Otherwise a new edge will be created.
@@ -219,7 +210,7 @@ class MultiDiGraph(MultiGraph,DiGraph):
         Examples
         --------
         The following all add the edge e=(1,2) to graph G:
-        
+
         >>> G = nx.MultiDiGraph()
         >>> e = (1,2)
         >>> G.add_edge(1, 2)           # explicit two-node form
@@ -255,7 +246,7 @@ class MultiDiGraph(MultiGraph,DiGraph):
             if key is None:
                 # find a unique integer key
                 # other methods might be better here?
-                key=0
+                key=len(keydict)
                 while key in keydict:
                     key+=1
             datadict=keydict.get(key,{})
@@ -272,15 +263,15 @@ class MultiDiGraph(MultiGraph,DiGraph):
             self.pred[v][u] = keydict
 
     def remove_edge(self, u, v, key=None):
-        """Remove an edge between u and v.  
+        """Remove an edge between u and v.
 
         Parameters
         ----------
-        u,v: nodes 
+        u,v: nodes
             Remove an edge between nodes u and v.
         key : hashable identifier, optional (default=None)
-            Used to distinguish multiple edges between a pair of nodes.  
-            If None remove a single (arbitrary) edge between u and v.
+            Used to distinguish multiple edges between a pair of nodes.
+            If None remove a single (abritrary) edge between u and v.
 
         Raises
         ------
@@ -294,7 +285,7 @@ class MultiDiGraph(MultiGraph,DiGraph):
 
         Examples
         --------
-        >>> G = nx.MultiDiGraph()  
+        >>> G = nx.MultiDiGraph()
         >>> G.add_path([0,1,2,3])
         >>> G.remove_edge(0,1)
         >>> e = (1,2)
@@ -302,13 +293,13 @@ class MultiDiGraph(MultiGraph,DiGraph):
 
         For multiple edges
 
-        >>> G = nx.MultiDiGraph()  
+        >>> G = nx.MultiDiGraph()
         >>> G.add_edges_from([(1,2),(1,2),(1,2)])
         >>> G.remove_edge(1,2) # remove a single (arbitrary) edge
-        
+
         For edges with keys
 
-        >>> G = nx.MultiDiGraph()   
+        >>> G = nx.MultiDiGraph()
         >>> G.add_edge(1,2,key='first')
         >>> G.add_edge(1,2,key='second')
         >>> G.remove_edge(1,2,key='second')
@@ -328,7 +319,7 @@ class MultiDiGraph(MultiGraph,DiGraph):
             except (KeyError):
                 raise NetworkXError(
                 "The edge %s-%s with key %s is not in the graph."%(u,v,key))
-        if len(d)==0: 
+        if len(d)==0:
             # remove the key entries if last edge
             del self.succ[u][v]
             del self.pred[v][u]
@@ -336,7 +327,7 @@ class MultiDiGraph(MultiGraph,DiGraph):
 
     def edges_iter(self, nbunch=None, data=False, keys=False):
         """Return an iterator over the edges.
-        
+
         Edges are returned as tuples with optional data and keys
         in the order (node, neighbor, key, data).
 
@@ -362,10 +353,11 @@ class MultiDiGraph(MultiGraph,DiGraph):
         Notes
         -----
         Nodes in nbunch that are not in the graph will be (quietly) ignored.
+        For directed graphs edges() is the same as out_edges().
 
         Examples
         --------
-        >>> G = nx.MultiDiGraph()  
+        >>> G = nx.MultiDiGraph()
         >>> G.add_path([0,1,2,3])
         >>> [e for e in G.edges_iter()]
         [(0, 1), (1, 2), (2, 3)]
@@ -378,21 +370,21 @@ class MultiDiGraph(MultiGraph,DiGraph):
 
         """
         if nbunch is None:
-            nodes_nbrs = self.adj.iteritems()
+            nodes_nbrs = iter(self.adj.items())
         else:
             nodes_nbrs=((n,self.adj[n]) for n in self.nbunch_iter(nbunch))
         if data:
             for n,nbrs in nodes_nbrs:
-                for nbr,keydict in nbrs.iteritems():
-                    for key,data in keydict.iteritems():
+                for nbr,keydict in nbrs.items():
+                    for key,data in keydict.items():
                         if keys:
                             yield (n,nbr,key,data)
                         else:
                             yield (n,nbr,data)
         else:
             for n,nbrs in nodes_nbrs:
-                for nbr,keydict in nbrs.iteritems():
-                    for key,data in keydict.iteritems():
+                for nbr,keydict in nbrs.items():
+                    for key,data in keydict.items():
                         if keys:
                             yield (n,nbr,key)
                         else:
@@ -401,9 +393,42 @@ class MultiDiGraph(MultiGraph,DiGraph):
     # alias out_edges to edges
     out_edges_iter=edges_iter
 
+    def out_edges(self, nbunch=None, keys=False, data=False):
+        """Return a list of the outgoing edges.
+
+        Edges are returned as tuples with optional data and keys
+        in the order (node, neighbor, key, data).
+
+        Parameters
+        ----------
+        nbunch : iterable container, optional (default= all nodes)
+            A container of nodes.  The container will be iterated
+            through once.
+        data : bool, optional (default=False)
+            If True, return edge attribute dict with each edge.
+        keys : bool, optional (default=False)
+            If True, return edge keys with each edge.
+
+        Returns
+        -------
+        out_edges : list
+            An listr of (u,v), (u,v,d) or (u,v,key,d) tuples of edges.
+
+        Notes
+        -----
+        Nodes in nbunch that are not in the graph will be (quietly) ignored.
+        For directed graphs edges() is the same as out_edges().
+
+        See Also
+        --------
+        in_edges: return a list of incoming edges
+        """
+        return list(self.out_edges_iter(nbunch, keys=keys, data=data))
+
+
     def in_edges_iter(self, nbunch=None, data=False, keys=False):
         """Return an iterator over the incoming edges.
-        
+
         Parameters
         ----------
         nbunch : iterable container, optional (default= all nodes)
@@ -424,52 +449,76 @@ class MultiDiGraph(MultiGraph,DiGraph):
         edges_iter : return an iterator of edges
         """
         if nbunch is None:
-            nodes_nbrs=self.pred.iteritems()
+            nodes_nbrs=iter(self.pred.items())
         else:
             nodes_nbrs=((n,self.pred[n]) for n in self.nbunch_iter(nbunch))
         if data:
             for n,nbrs in nodes_nbrs:
-                for nbr,keydict in nbrs.iteritems():
-                    for key,data in keydict.iteritems():
+                for nbr,keydict in nbrs.items():
+                    for key,data in keydict.items():
                         if keys:
                             yield (nbr,n,key,data)
                         else:
                             yield (nbr,n,data)
         else:
             for n,nbrs in nodes_nbrs:
-                for nbr,keydict in nbrs.iteritems():
-                    for key,data in keydict.iteritems():
+                for nbr,keydict in nbrs.items():
+                    for key,data in keydict.items():
                         if keys:
                             yield (nbr,n,key)
                         else:
                             yield (nbr,n)
 
+    def in_edges(self, nbunch=None, keys=False, data=False):
+        """Return a list of the incoming edges.
+
+        Parameters
+        ----------
+        nbunch : iterable container, optional (default= all nodes)
+            A container of nodes.  The container will be iterated
+            through once.
+        data : bool, optional (default=False)
+            If True, return edge attribute dict with each edge.
+        keys : bool, optional (default=False)
+            If True, return edge keys with each edge.
+
+        Returns
+        -------
+        in_edges : list
+            A list  of (u,v), (u,v,d) or (u,v,key,d) tuples of edges.
+
+        See Also
+        --------
+        out_edges: return a list of outgoing edges
+        """
+        return list(self.in_edges_iter(nbunch, keys=keys, data=data))
+
 
     def degree_iter(self, nbunch=None, weighted=False):
-        """Return an iterator for (node, degree). 
+        """Return an iterator for (node, degree).
 
-        The node degree is the number of edges adjacent to the node. 
+        The node degree is the number of edges adjacent to the node.
 
         Parameters
         ----------
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
-            through once.    
+            through once.
         weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.  
+           If True return the sum of edge weights adjacent to the node.
 
         Returns
         -------
-        nd_iter : an iterator 
+        nd_iter : an iterator
             The iterator returns two-tuples of (node, degree).
-        
+
         See Also
         --------
         degree
 
         Examples
         --------
-        >>> G = nx.MultiDiGraph() 
+        >>> G = nx.MultiDiGraph()
         >>> G.add_path([0,1,2,3])
         >>> list(G.degree_iter(0)) # node 0 with degree 1
         [(0, 1)]
@@ -477,39 +526,31 @@ class MultiDiGraph(MultiGraph,DiGraph):
         [(0, 1), (1, 2)]
 
         """
-        from itertools import izip
         if nbunch is None:
-            nodes_nbrs=izip(self.succ.iteritems(),self.pred.iteritems())
+            nodes_nbrs=zip(iter(self.succ.items()),iter(self.pred.items()))
         else:
-            nodes_nbrs=izip(
+            nodes_nbrs=zip(
                 ((n,self.succ[n]) for n in self.nbunch_iter(nbunch)),
                 ((n,self.pred[n]) for n in self.nbunch_iter(nbunch)))
-
         if weighted:
         # edge weighted graph - degree is sum of nbr edge weights
             for (n,succ),(n2,pred) in nodes_nbrs:
-                deg = sum([d.get('weight',1) 
-                           for data in pred.itervalues()
-                           for d in data.itervalues()])
+                deg = sum([d.get('weight',1)
+                           for data in pred.values()
+                           for d in data.values()])
                 deg += sum([d.get('weight',1) 
-                           for data in succ.itervalues()
-                           for d in data.itervalues()])
-#                if n in succ:
-                    # double counted selfloop so subtract
-#                    deg -= sum([d.get('weight',1) 
-#                           for key,d in succ[n].iteritems()])
+                           for data in succ.values()
+                           for d in data.values()])
                 yield (n, deg)
         else:
             for (n,succ),(n2,pred) in nodes_nbrs:
-                indeg = sum([len(data) for data in pred.itervalues()])
-                outdeg = sum([len(data) for data in succ.itervalues()])
+                indeg = sum([len(data) for data in pred.values()])
+                outdeg = sum([len(data) for data in succ.values()])
                 yield (n, indeg + outdeg)  
-# double counted selfloop so subtract
-#                      - (n in succ and len(succ[n])))
 
 
     def in_degree_iter(self, nbunch=None, weighted=False):
-        """Return an iterator for (node, in-degree). 
+        """Return an iterator for (node, in-degree).
 
         The node in-degree is the number of edges pointing in to the node.
 
@@ -517,22 +558,22 @@ class MultiDiGraph(MultiGraph,DiGraph):
         ----------
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
-            through once.    
+            through once.
         weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.  
+           If True return the sum of edge weights adjacent to the node.
 
         Returns
         -------
-        nd_iter : an iterator 
+        nd_iter : an iterator
             The iterator returns two-tuples of (node, in-degree).
-        
+
         See Also
         --------
         degree, in_degree, out_degree, out_degree_iter
 
         Examples
         --------
-        >>> G = nx.MultiDiGraph()  
+        >>> G = nx.MultiDiGraph()
         >>> G.add_path([0,1,2,3])
         >>> list(G.in_degree_iter(0)) # node 0 with degree 0
         [(0, 0)]
@@ -541,24 +582,24 @@ class MultiDiGraph(MultiGraph,DiGraph):
 
         """
         if nbunch is None:
-            nodes_nbrs=self.pred.iteritems()
+            nodes_nbrs=iter(self.pred.items())
         else:
             nodes_nbrs=((n,self.pred[n]) for n in self.nbunch_iter(nbunch))
 
         if weighted:
             # edge weighted graph - degree is sum of nbr edge weights
             for n,pred in nodes_nbrs:
-                deg = sum([d.get('weight',1) 
-                           for data in pred.itervalues()
-                           for d in data.itervalues()])
+                deg = sum([d.get('weight',1)
+                           for data in pred.values()
+                           for d in data.values()])
                 yield (n, deg)
         else:
             for n,nbrs in nodes_nbrs:
-                yield (n, sum([len(data) for data in nbrs.itervalues()]) )
+                yield (n, sum([len(data) for data in nbrs.values()]) )
 
 
     def out_degree_iter(self, nbunch=None, weighted=False):
-        """Return an iterator for (node, out-degree). 
+        """Return an iterator for (node, out-degree).
 
         The node out-degree is the number of edges pointing out of the node.
 
@@ -566,15 +607,15 @@ class MultiDiGraph(MultiGraph,DiGraph):
         ----------
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
-            through once.    
+            through once.
         weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.  
+           If True return the sum of edge weights adjacent to the node.
 
         Returns
         -------
-        nd_iter : an iterator 
+        nd_iter : an iterator
             The iterator returns two-tuples of (node, out-degree).
-        
+
         See Also
         --------
         degree, in_degree, out_degree, in_degree_iter
@@ -590,19 +631,19 @@ class MultiDiGraph(MultiGraph,DiGraph):
 
         """
         if nbunch is None:
-            nodes_nbrs=self.succ.iteritems()
+            nodes_nbrs=iter(self.succ.items())
         else:
             nodes_nbrs=((n,self.succ[n]) for n in self.nbunch_iter(nbunch))
 
         if weighted:
             for n,succ in nodes_nbrs:
-                deg = sum([d.get('weight',1) 
-                           for data in succ.itervalues()
-                           for d in data.itervalues()])
+                deg = sum([d.get('weight',1)
+                           for data in succ.values()
+                           for d in data.values()])
                 yield (n, deg)
         else:
             for n,nbrs in nodes_nbrs:
-                yield (n, sum([len(data) for data in nbrs.itervalues()]) )
+                yield (n, sum([len(data) for data in nbrs.values()]) )
 
     def is_multigraph(self):
         """Return True if graph is a multigraph, False otherwise."""
@@ -614,7 +655,7 @@ class MultiDiGraph(MultiGraph,DiGraph):
 
     def to_directed(self):
         """Return a directed copy of the graph.
- 
+
         Returns
         -------
         G : MultiDiGraph
@@ -628,12 +669,12 @@ class MultiDiGraph(MultiGraph,DiGraph):
         in the (arbitrary) order that the edges are encountered.  For
         more customized control of the edge attributes use add_edge().
 
-        This returns a "deepcopy" of the edge, node, and 
+        This returns a "deepcopy" of the edge, node, and
         graph attributes which attempts to completely copy
         all of the data and references.
 
-        This is in contrast to the similar G=DiGraph(D) which returns a 
-        shallow copy of the data. 
+        This is in contrast to the similar G=DiGraph(D) which returns a
+        shallow copy of the data.
 
         See the Python copy module for more information on shallow
         and deep copies, http://docs.python.org/library/copy.html.
@@ -648,7 +689,7 @@ class MultiDiGraph(MultiGraph,DiGraph):
 
         If already directed, return a (deep) copy
 
-        >>> G = nx.MultiDiGraph()   
+        >>> G = nx.MultiDiGraph()
         >>> G.add_path([0,1])
         >>> H = G.to_directed()
         >>> H.edges()
@@ -656,13 +697,19 @@ class MultiDiGraph(MultiGraph,DiGraph):
         """
         return deepcopy(self)
 
-    def to_undirected(self):
+    def to_undirected(self, reciprocal=False):
         """Return an undirected representation of the digraph.
- 
+
+        Parameters
+        ----------
+        reciprocal : bool (optional)
+          If True only keep edges that appear in both directions 
+          in the original digraph. 
+
         Returns
         -------
         G : MultiGraph
-            An undirected graph with the same name and nodes and 
+            An undirected graph with the same name and nodes and
             with edge (u,v,data) if either (u,v,data) or (v,u,data)
             is in the digraph.  If both edges exist in digraph and
             their edge data is different, only one edge is created
@@ -671,12 +718,12 @@ class MultiDiGraph(MultiGraph,DiGraph):
 
         Notes
         -----
-        This returns a "deepcopy" of the edge, node, and 
+        This returns a "deepcopy" of the edge, node, and
         graph attributes which attempts to completely copy
         all of the data and references.
 
-        This is in contrast to the similar D=DiGraph(G) which returns a 
-        shallow copy of the data. 
+        This is in contrast to the similar D=DiGraph(G) which returns a
+        shallow copy of the data.
 
         See the Python copy module for more information on shallow
         and deep copies, http://docs.python.org/library/copy.html.
@@ -684,10 +731,17 @@ class MultiDiGraph(MultiGraph,DiGraph):
         H=MultiGraph()
         H.name=self.name
         H.add_nodes_from(self)
-        H.add_edges_from( ((u,v,key,deepcopy(data))
-                           for u,nbrs in self.adjacency_iter()
-                           for v,keydict in nbrs.iteritems()
-                           for key,data in keydict.items()))
+        if reciprocal is True:
+            H.add_edges_from( (u,v,key,deepcopy(data))
+                              for u,nbrs in self.adjacency_iter()
+                              for v,keydict in nbrs.items()
+                              for key,data in keydict.items()
+                              if self.has_edge(v,u,key))
+        else:
+            H.add_edges_from( (u,v,key,deepcopy(data))
+                              for u,nbrs in self.adjacency_iter()
+                              for v,keydict in nbrs.items()
+                              for key,data in keydict.items())
         H.graph=deepcopy(self.graph)
         H.node=deepcopy(self.node)
         return H
@@ -695,18 +749,18 @@ class MultiDiGraph(MultiGraph,DiGraph):
     def subgraph(self, nbunch):
         """Return the subgraph induced on nodes in nbunch.
 
-        The induced subgraph of the graph contains the nodes in nbunch 
-        and the edges between those nodes.  
-        
+        The induced subgraph of the graph contains the nodes in nbunch
+        and the edges between those nodes.
+
         Parameters
         ----------
         nbunch : list, iterable
-            A container of nodes which will be iterated through once.    
+            A container of nodes which will be iterated through once.
 
         Returns
         -------
         G : Graph
-            A subgraph of the graph with the same edge attributes.  
+            A subgraph of the graph with the same edge attributes.
 
         Notes
         -----
@@ -720,21 +774,20 @@ class MultiDiGraph(MultiGraph,DiGraph):
         If edge attributes are containers, a deep copy can be obtained using:
         G.subgraph(nbunch).copy()
 
-        For an in-place reduction of a graph to a subgraph you can remove nodes:
-        G.remove_nodes_from([ n in G if n not in set(nbunch)])  
+        For an inplace reduction of a graph to a subgraph you can remove nodes:
+        G.remove_nodes_from([ n in G if n not in set(nbunch)])
 
         Examples
         --------
         >>> G = nx.Graph()   # or DiGraph, MultiGraph, MultiDiGraph, etc
         >>> G.add_path([0,1,2,3])
         >>> H = G.subgraph([0,1,2])
-        >>> print H.edges()
+        >>> H.edges()
         [(0, 1), (1, 2)]
         """
         bunch = self.nbunch_iter(nbunch)
-        # create new graph and copy subgraph into it       
+        # create new graph and copy subgraph into it
         H = self.__class__()
-        H.name = "Subgraph of (%s)"%(self.name)
         # namespace shortcuts for speed
         H_succ=H.succ
         H_pred=H.pred
@@ -747,7 +800,7 @@ class MultiDiGraph(MultiGraph,DiGraph):
         # add edges
         for u in H_succ:
             Hnbrs=H_succ[u]
-            for v,edgedict in self_succ[u].iteritems():
+            for v,edgedict in self_succ[u].items():
                 if v in H_succ:
                     # add both representations of edge: u-v and v-u
                     # they share the same edgedict
