@@ -34,26 +34,24 @@ nx_pygraphviz, nx_pydot
 __author__ = """\n""".join(['Aric Hagberg (hagberg@lanl.gov)',
                            'Pieter Swart (swart@lanl.gov)',
                            'Dan Schult(dschult@colgate.edu)'])
-#    Copyright (C) 2006-2010 by 
+#    Copyright (C) 2006-2011 by 
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
 
+import warnings
+import networkx as nx
 
-__all__ = ['to_networkx_graph','from_whatever', 
-           'convert_node_labels_to_integers', 'relabel_nodes',
+__all__ = ['to_networkx_graph',
            'from_dict_of_dicts', 'to_dict_of_dicts',
            'from_dict_of_lists', 'to_dict_of_lists',
            'from_edgelist', 'to_edgelist',
            'from_numpy_matrix', 'to_numpy_matrix',
+           'to_numpy_recarray',
            'from_scipy_sparse_matrix', 'to_scipy_sparse_matrix']
 
-import warnings
-import networkx as nx
-
-
 def _prep_create_using(create_using):
     """Return a graph object ready to be populated.
 
@@ -117,19 +115,17 @@ def to_networkx_graph(data,create_using=None,multigraph_input=False):
             if hasattr(data,'graph') and isinstance(data.graph,dict):
                 result.graph=data.graph.copy()
             if hasattr(data,'node') and isinstance(data.node,dict):
-                result.node=dict( (n,dd.copy()) for n,dd in data.node.iteritems() )
+                result.node=dict( (n,dd.copy()) for n,dd in data.node.items() )
             return result
         except:
-            raise nx.NetworkXError,\
-                "Input is not a correct NetworkX graph."
+            raise nx.NetworkXError("Input is not a correct NetworkX graph.")
 
     # pygraphviz  agraph
     if hasattr(data,"is_strict"):
         try:
             return nx.from_agraph(data,create_using=create_using)
         except:
-            raise nx.NetworkXError,\
-                  "Input is not a correct pygraphviz graph."
+            raise nx.NetworkXError("Input is not a correct pygraphviz graph.")
 
     # dict of dicts/lists
     if isinstance(data,dict):
@@ -143,12 +139,13 @@ def to_networkx_graph(data,create_using=None,multigraph_input=False):
                 raise TypeError("Input is not known type.")
 
     # list or generator of edges
-    if isinstance(data,list) or hasattr(data,'next'): 
+    if (isinstance(data,list)
+        or hasattr(data,'next')
+        or hasattr(data, '__next__')): 
         try:
             return from_edgelist(data,create_using=create_using)
         except:
-            raise nx.NetworkXError,\
-                  "Input is not a valid edge list"
+            raise nx.NetworkXError("Input is not a valid edge list")
 
     # numpy matrix or ndarray 
     try:
@@ -158,8 +155,8 @@ def to_networkx_graph(data,create_using=None,multigraph_input=False):
             try:
                 return from_numpy_matrix(data,create_using=create_using)
             except:
-                raise nx.NetworkXError,\
-                  "Input is not a correct numpy matrix or array."
+                raise nx.NetworkXError(\
+                  "Input is not a correct numpy matrix or array.")
     except ImportError:
         warnings.warn('numpy not found, skipping conversion test.',
                       ImportWarning)
@@ -171,31 +168,18 @@ def to_networkx_graph(data,create_using=None,multigraph_input=False):
             try:
                 return from_scipy_sparse_matrix(data,create_using=create_using)
             except:
-                raise nx.NetworkXError, \
-                      "Input is not a correct scipy sparse matrix type."
+                raise nx.NetworkXError(\
+                      "Input is not a correct scipy sparse matrix type.")
     except ImportError:
         warnings.warn('scipy not found, skipping conversion test.',
                       ImportWarning)
 
 
-    raise nx.NetworkXError, \
-          "Input is not a known data type for conversion."
+    raise nx.NetworkXError(\
+          "Input is not a known data type for conversion.")
 
     return 
 
-    
-
-def from_whatever(data,create_using=None,multigraph_input=False):
-    """Deprecated. Use to_networkx_graph.
-
-    See Also
-    --------
-    to_networkx_graph()
-    """
-
-    return to_networkx_graph(data,
-                             create_using=create_using,
-                             multigraph_input=multigraph_input)
 
 def convert_to_undirected(G):
     """Return a new undirected representation of the graph G.
@@ -211,155 +195,8 @@ def convert_to_directed(G):
     return G.to_directed()
 
 
-def relabel_nodes(G,mapping):
-    """Return a copy of G with node labels transformed by mapping.
-
-    Parameters
-    ----------
-    G : graph
-       A NetworkX graph 
-
-    mapping : dictionary or function
-       Either a dictionary with the old labels as keys and new labels as values
-       or a function transforming an old label with a new label.
-       In either case, the new labels must be hashable Python objects.
-
-    Examples
-    --------
-    mapping as dictionary
-
-    >>> G=nx.path_graph(3)  # nodes 0-1-2
-    >>> mapping={0:'a',1:'b',2:'c'}
-    >>> H=nx.relabel_nodes(G,mapping)
-    >>> print H.nodes()
-    ['a', 'c', 'b']
-
-    >>> G=nx.path_graph(26) # nodes 0..25
-    >>> mapping=dict(zip(G.nodes(),"abcdefghijklmnopqrstuvwxyz"))
-    >>> H=nx.relabel_nodes(G,mapping) # nodes a..z
-    >>> mapping=dict(zip(G.nodes(),xrange(1,27)))
-    >>> G1=nx.relabel_nodes(G,mapping) # nodes 1..26
-
-    mapping as function
-
-    >>> G=nx.path_graph(3)
-    >>> def mapping(x):
-    ...    return x**2
-    >>> H=nx.relabel_nodes(G,mapping)
-    >>> print H.nodes()
-    [0, 1, 4]
-
-    See Also
-    --------
-    convert_node_labels_to_integers()
-
-    """
-    H=G.__class__()
-    H.name="(%s)" % G.name
-
-    if hasattr(mapping,"__getitem__"):   # if we are a dict
-        map_func=mapping.__getitem__   # call as a function
-    else:
-        map_func=mapping
-
-    for node in G:
-        try:
-            H.add_node(map_func(node))
-        except:
-            raise nx.NetworkXError,\
-                  "relabeling function cannot be applied to node %s" % node
-
-    #for n1,n2,d in G.edges_iter(data=True):
-    #    u=map_func(n1)
-    #    v=map_func(n2)
-    #    H.add_edge(u,v,d)
-    if G.is_multigraph():
-        H.add_edges_from( (map_func(n1),map_func(n2),k,d) 
-                          for (n1,n2,k,d) in G.edges_iter(keys=True,data=True)) 
-    else:
-        H.add_edges_from( (map_func(n1),map_func(n2),d) 
-                          for (n1,n2,d) in G.edges_iter(data=True)) 
-
-    H.node.update(dict((map_func(n),d) for n,d in G.node.iteritems()))
-    H.graph.update(G.graph)
-
-    return H        
-    
-
-def convert_node_labels_to_integers(G,first_label=0,
-                                    ordering="default",
-                                    discard_old_labels=True):
-    """ Return a copy of G node labels replaced with integers.
-
-    Parameters
-    ----------
-    G : graph
-       A NetworkX graph 
-
-    first_label : int, optional (default=0)       
-       An integer specifying the offset in numbering nodes.
-       The n new integer labels are numbered first_label, ..., n+first_label.
-
-    ordering : string
-        "default" : inherit node ordering from G.nodes() 
-        "sorted"  : inherit node ordering from sorted(G.nodes())
-        "increasing degree" : nodes are sorted by increasing degree
-        "decreasing degree" : nodes are sorted by decreasing degree
-
-    discard_old_labels : bool, optional (default=True)
-       if True (default) discard old labels
-       if False, create a dict self.node_labels that maps new
-       labels to old labels
-    """
-#    This function strips information attached to the nodes and/or
-#    edges of a graph, and returns a graph with appropriate integer
-#    labels. One can view this as a re-labeling of the nodes. Be
-#    warned that the term "labeled graph" has a loaded meaning
-#    in graph theory. The fundamental issue is whether the names
-#    (labels) of the nodes (and edges) matter in deciding when two
-#    graphs are the same. For example, in problems of graph enumeration
-#    there is a distinct difference in techniques required when
-#    counting labeled vs. unlabeled graphs.
-
-#    When implementing graph
-#    algorithms it is often convenient to strip off the original node
-#    and edge information and appropriately relabel the n nodes with
-#    the integer values 1,..,n. This is the purpose of this function,
-#    and it provides the option (see discard_old_labels variable) to either
-#    preserve the original labels in separate dicts (these are not
-#    returned but made an attribute of the new graph.
-
-    N=G.number_of_nodes()+first_label
-    if ordering=="default":
-        mapping=dict(zip(G.nodes(),range(first_label,N)))
-    elif ordering=="sorted":
-        nlist=G.nodes()
-        nlist.sort()
-        mapping=dict(zip(nlist,range(first_label,N)))
-    elif ordering=="increasing degree":
-        dv_pairs=[(d,n) for (n,d) in G.degree_iter()]
-        dv_pairs.sort() # in-place sort from lowest to highest degree
-        mapping=dict(zip([n for d,n in dv_pairs],range(first_label,N)))
-    elif ordering=="decreasing degree":
-        dv_pairs=[(d,n) for (n,d) in G.degree_iter()]
-        dv_pairs.sort() # in-place sort from lowest to highest degree
-        dv_pairs.reverse()
-        mapping=dict(zip([n for d,n in dv_pairs],range(first_label,N)))
-    else:
-        raise nx.NetworkXError,\
-              "unknown value of node ordering variable: ordering"
-
-    H=relabel_nodes(G,mapping)
-
-    H.name="("+G.name+")_with_int_labels"
-    if not discard_old_labels:
-        H.node_labels=mapping
-    return H
-
-
-
 def to_dict_of_lists(G,nodelist=None):
-    """Return adjacency representation of graph as a dictionary of lists
+    """Return adjacency representation of graph as a dictionary of lists.
 
     Parameters
     ----------
@@ -404,25 +241,24 @@ def from_dict_of_lists(d,create_using=None):
     """
     G=_prep_create_using(create_using)
     G.add_nodes_from(d)        
-
     if G.is_multigraph() and not G.is_directed():
         # a dict_of_lists can't show multiedges.  BUT for undirected graphs,
         # each edge shows up twice in the dict_of_lists.  
         # So we need to treat this case separately.
         seen={}
-        for node,nbrlist in d.iteritems():
+        for node,nbrlist in d.items():
             for nbr in nbrlist:
                 if nbr not in seen:
                     G.add_edge(node,nbr)
             seen[node]=1  # don't allow reverse edge to show up 
     else:
-        G.add_edges_from( ((node,nbr) for node,nbrlist in d.iteritems() 
+        G.add_edges_from( ((node,nbr) for node,nbrlist in d.items() 
                            for nbr in nbrlist) )
     return G                         
 
 
 def to_dict_of_dicts(G,nodelist=None,edge_data=None):
-    """Return adjacency representation of graph as a dictionary of dictionaries
+    """Return adjacency representation of graph as a dictionary of dictionaries.
 
     Parameters
     ----------
@@ -452,7 +288,7 @@ def to_dict_of_dicts(G,nodelist=None,edge_data=None):
         if edge_data is None:
             for u in nodelist:
                 dod[u]={}
-                for v,data in ((v,data) for v,data in G[u].iteritems() if v in nodelist):
+                for v,data in ((v,data) for v,data in G[u].items() if v in nodelist):
                     dod[u][v]=data
         else: # nodelist and edge_data are not None
             for u in nodelist:
@@ -488,55 +324,56 @@ def from_dict_of_dicts(d,create_using=None,multigraph_input=False):
     """
     G=_prep_create_using(create_using)
     G.add_nodes_from(d)
-
     # is dict a MultiGraph or MultiDiGraph?
     if multigraph_input:
         # make a copy of the list of edge data (but not the edge data)
         if G.is_directed():  
             if G.is_multigraph():
                 G.add_edges_from( (u,v,key,data)
-                                  for u,nbrs in d.iteritems() 
-                                  for v,datadict in nbrs.iteritems() 
+                                  for u,nbrs in d.items() 
+                                  for v,datadict in nbrs.items() 
                                   for key,data in datadict.items()
                                 )
             else:
                 G.add_edges_from( (u,v,data)
-                                  for u,nbrs in d.iteritems() 
-                                  for v,datadict in nbrs.iteritems() 
+                                  for u,nbrs in d.items() 
+                                  for v,datadict in nbrs.items() 
                                   for key,data in datadict.items()
                                 )
         else: # Undirected
             if G.is_multigraph():
                 seen=set()   # don't add both directions of undirected graph
-                for u,nbrs in d.iteritems():
-                    for v,datadict in nbrs.iteritems():
-                        if v not in seen:
+                for u,nbrs in d.items():
+                    for v,datadict in nbrs.items():
+                        if (u,v) not in seen:
                             G.add_edges_from( (u,v,key,data) 
                                                for key,data in datadict.items()
-                                          )
-                    seen.add(u) 
+                                              )
+                            seen.add((v,u)) 
             else:
                 seen=set()   # don't add both directions of undirected graph
-                for u,nbrs in d.iteritems():
-                    for v,datadict in nbrs.iteritems():
-                        if v not in seen:
+                for u,nbrs in d.items():
+                    for v,datadict in nbrs.items():
+                        if (u,v) not in seen:
                             G.add_edges_from( (u,v,data)
                                         for key,data in datadict.items() )
-                    seen.add(u) 
+                            seen.add((v,u)) 
 
     else: # not a multigraph to multigraph transfer
-        if G.is_directed():
-            G.add_edges_from( ( (u,v,data) 
-                                for u,nbrs in d.iteritems() 
-                                for v,data in nbrs.iteritems()) )
-        # need this if G is multigraph and slightly faster if not multigraph
-        else:
+        if G.is_multigraph() and not G.is_directed():
+            # d can have both representations u-v, v-u in dict.  Only add one.
+            # We don't need this check for digraphs since we add both directions,
+            # or for Graph() since it is done implicitly (parallel edges not allowed)
             seen=set()
-            for u,nbrs in d.iteritems():
-                for v,data in nbrs.iteritems():
-                    if v not in seen:
+            for u,nbrs in d.items():
+                for v,data in nbrs.items():
+                    if (u,v) not in seen:
                         G.add_edge(u,v,attr_dict=data)
-                seen.add(u)
+                    seen.add((v,u))
+        else:
+            G.add_edges_from( ( (u,v,data) 
+                                for u,nbrs in d.items() 
+                                for v,data in nbrs.items()) )
     return G                         
 
 def to_edgelist(G,nodelist=None):
@@ -580,7 +417,8 @@ def from_edgelist(edgelist,create_using=None):
     G.add_edges_from(edgelist)
     return G                         
 
-def to_numpy_matrix(G,nodelist=None,dtype=None,order=None):
+def to_numpy_matrix(G, nodelist=None, dtype=None, order=None,
+                    multigraph_weight=sum, weight='weight'):
     """Return the graph adjacency matrix as a NumPy matrix.
 
     Parameters
@@ -592,24 +430,37 @@ def to_numpy_matrix(G,nodelist=None,dtype=None,order=None):
        The rows and columns are ordered according to the nodes in `nodelist`.
        If `nodelist` is None, then the ordering is produced by G.nodes().
 
-    dtype : NumPy data-type, optional
-        A valid NumPy dtype used to initialize the array. If None, then the
-        NumPy default is used.
+    dtype : NumPy data type, optional
+        A valid single NumPy data type used to initialize the array. 
+        This must be a simple type such as int or numpy.float64 and
+        not a compound data type (see to_numpy_recarray)
+        If None, then the NumPy default is used.
 
     order : {'C', 'F'}, optional
         Whether to store multidimensional data in C- or Fortran-contiguous
         (row- or column-wise) order in memory. If None, then the NumPy default 
         is used.
 
+    multigraph_weight : {sum, min, max}, optional        
+        An operator that determines how weights in multigraphs are handled.
+        The default is to sum the weights of the multiple edges.
+
+    weight: string, optional       
+       Edge data key corresponding to the edge weight.
+
     Returns
     -------
     M : NumPy matrix
        Graph adjacency matrix.
 
+    See Also
+    --------
+    to_numpy_recarray, from_numpy_matrix
+
     Notes
     -----
-    The matrix entries are populated using the 'weight' edge attribute. When
-    an edge does not have the 'weight' attribute, the value of the entry is 1.
+    The matrix entries are assigned with weight edge attribute. When
+    an edge does not have the weight attribute, the value of the entry is 1.
     For multiple edges, the values of the entries are the sums of the edge
     attributes for each edge.
 
@@ -632,8 +483,204 @@ def to_numpy_matrix(G,nodelist=None,dtype=None,order=None):
     try:
         import numpy as np
     except ImportError:
-        raise ImportError, \
-          "to_numpy_matrix() requires numpy: http://scipy.org/ "
+        raise ImportError(\
+          "to_numpy_matrix() requires numpy: http://scipy.org/ ")
+
+    if nodelist is None:
+        nodelist = G.nodes()
+
+    nodeset = set(nodelist)
+    if len(nodelist) != len(nodeset):
+        msg = "Ambiguous ordering: `nodelist` contained duplicates."
+        raise nx.NetworkXError(msg)
+
+    nlen=len(nodelist)
+    undirected = not G.is_directed()
+    index=dict(zip(nodelist,range(nlen)))
+
+    if G.is_multigraph():
+        # Handle MultiGraphs and MultiDiGraphs
+        # array of nan' to start with, any leftover nans will be converted to 0
+        # nans are used so we can use sum, min, max for multigraphs
+        M = np.zeros((nlen,nlen), dtype=dtype, order=order)+np.nan
+        # use numpy nan-aware operations
+        operator={sum:np.nansum, min:np.nanmin, max:np.nanmax}
+        try:
+            op=operator[multigraph_weight]
+        except:
+            raise ValueError('multigraph_weight must be sum, min, or max')
+
+        for u,v,attrs in G.edges_iter(data=True):
+            if (u in nodeset) and (v in nodeset):
+                i,j = index[u],index[v]
+                e_weight = attrs.get(weight, 1)
+                M[i,j] = op([e_weight,M[i,j]]) 
+                if undirected:
+                    M[j,i] = M[i,j]
+        # convert any nans to zeros
+        M = np.asmatrix(np.nan_to_num(M))
+    else:
+        # Graph or DiGraph, this is much faster than above 
+        M = np.zeros((nlen,nlen), dtype=dtype, order=order)
+        for u,nbrdict in G.adjacency_iter():
+            for v,d in nbrdict.items():
+                try:
+                    M[index[u],index[v]]=d.get(weight,1)
+                except KeyError:
+                    pass
+        M = np.asmatrix(M)
+    return M
+
+
+def from_numpy_matrix(A,create_using=None):
+    """Return a graph from numpy matrix.
+
+    The numpy matrix is interpreted as an adjacency matrix for the graph.
+
+    Parameters
+    ----------
+    A : numpy matrix
+      An adjacency matrix representation of a graph
+
+    create_using : NetworkX graph
+       Use specified graph for result.  The default is Graph()
+
+    Notes
+    -----
+    If the numpy matrix has a single data type for each matrix entry it 
+    will be converted to an appropriate Python data type.  
+
+    If the numpy matrix has a user-specified compound data type the names
+    of the data fields will be used as attribute keys in the resulting 
+    NetworkX graph.
+
+    See Also
+    --------
+    to_numpy_matrix, to_numpy_recarray
+
+    Examples
+    --------
+    Simple integer weights on edges:
+
+    >>> import numpy
+    >>> A=numpy.matrix([[1,1],[2,1]])
+    >>> G=nx.from_numpy_matrix(A)
+
+    User defined compound data type on edges:
+
+    >>> import numpy
+    >>> dt=[('weight',float),('cost',int)]
+    >>> A=numpy.matrix([[(1.0,2)]],dtype=dt)                      
+    >>> G=nx.from_numpy_matrix(A)
+    >>> G.edges(data=True)
+    [(0, 0, {'cost': 2, 'weight': 1.0})]
+    """
+    kind_to_python_type={'f':float,
+                         'i':int,
+                         'u':int,
+                         'b':bool,
+                         'c':complex,
+                         'S':str,
+                         'V':'void'}
+
+    try: # Python 3.x
+        blurb = chr(1245) # just to trigger the exception
+        kind_to_python_type['U']=str
+    except ValueError: # Python 2.6+
+        kind_to_python_type['U']=unicode
+
+    # This should never fail if you have created a numpy matrix with numpy...  
+    try:
+        import numpy as np
+    except ImportError:
+        raise ImportError(\
+          "from_numpy_matrix() requires numpy: http://scipy.org/ ")
+
+    G=_prep_create_using(create_using)
+    n,m=A.shape
+    if n!=m:
+        raise nx.NetworkXError("Adjacency matrix is not square.",
+                               "nx,ny=%s"%(A.shape,))
+    dt=A.dtype
+    try:
+        python_type=kind_to_python_type[dt.kind]
+    except:
+        raise TypeError("Unknown numpy data type: %s"%dt)
+
+    # make sure we get isolated nodes
+    G.add_nodes_from(range(n)) 
+    # get a list of edges
+    x,y=np.asarray(A).nonzero()         
+
+    # handle numpy constructed data type
+    if python_type is 'void':
+        fields=sorted([(offset,dtype,name) for name,(dtype,offset) in
+                       A.dtype.fields.items()])
+        for (u,v) in zip(x,y):         
+            attr={}
+            for (offset,dtype,name),val in zip(fields,A[u,v]):
+                attr[name]=kind_to_python_type[dtype.kind](val)
+            G.add_edge(u,v,attr)
+    else: # basic data type
+        G.add_edges_from( ((u,v,{'weight':python_type(A[u,v])}) 
+                           for (u,v) in zip(x,y)) )
+    return G
+
+
+def to_numpy_recarray(G,nodelist=None,
+                      dtype=[('weight',float)],
+                      order=None):
+    """Return the graph adjacency matrix as a NumPy recarray.
+
+    Parameters
+    ----------
+    G : graph
+        The NetworkX graph used to construct the NumPy matrix.
+
+    nodelist : list, optional       
+       The rows and columns are ordered according to the nodes in `nodelist`.
+       If `nodelist` is None, then the ordering is produced by G.nodes().
+
+    dtype : NumPy data-type, optional
+        A valid NumPy named dtype used to initialize the NumPy recarray. 
+        The data type names are assumed to be keys in the graph edge attribute 
+        dictionary.
+
+    order : {'C', 'F'}, optional
+        Whether to store multidimensional data in C- or Fortran-contiguous
+        (row- or column-wise) order in memory. If None, then the NumPy default 
+        is used.
+
+    Returns
+    -------
+    M : NumPy recarray
+       The graph with specified edge data as a Numpy recarray 
+
+    Notes
+    -----
+    When `nodelist` does not contain every node in `G`, the matrix is built 
+    from the subgraph of `G` that is induced by the nodes in `nodelist`.
+    
+    Examples
+    --------
+    >>> G = nx.Graph()
+    >>> G.add_edge(1,2,weight=7.0,cost=5)
+    >>> A=nx.to_numpy_recarray(G,dtype=[('weight',float),('cost',int)])
+    >>> print(A.weight)
+    [[ 0.  7.]
+     [ 7.  0.]]
+    >>> print(A.cost)
+    [[0 5]
+     [5 0]]
+    """
+    try:
+        import numpy as np
+    except ImportError:
+        raise ImportError(\
+          "to_numpy_matrix() requires numpy: http://scipy.org/ ")
+
+    if G.is_multigraph():
+        raise nx.NetworkXError("Not implemented for multigraphs.")
 
     if nodelist is None:
         nodelist = G.nodes()
@@ -648,56 +695,16 @@ def to_numpy_matrix(G,nodelist=None,dtype=None,order=None):
     index=dict(zip(nodelist,range(nlen)))
     M = np.zeros((nlen,nlen), dtype=dtype, order=order)
 
+    names=M.dtype.names
     for u,v,attrs in G.edges_iter(data=True):
         if (u in nodeset) and (v in nodeset):
             i,j = index[u],index[v]
-            M[i,j] += attrs.get('weight', 1)
+            values=tuple([attrs[n] for n in names])
+            M[i,j] = values
             if undirected:
                 M[j,i] = M[i,j]
 
-    M = np.asmatrix(M)
-    return M
-
-def from_numpy_matrix(A,create_using=None):
-    """Return a graph from numpy matrix adjacency list. 
-
-    Parameters
-    ----------
-    A : numpy matrix
-      An adjacency matrix representation of a graph
-
-    create_using : NetworkX graph
-       Use specified graph for result.  The default is Graph()
-
-    Examples
-    --------
-    >>> import numpy
-    >>> A=numpy.matrix([[1,1],[2,1]])
-    >>> G=nx.from_numpy_matrix(A)
-
-    """
-    # This should never fail if you have created a numpy matrix with numpy...  
-    try:
-        import numpy as np
-    except ImportError:
-        raise ImportError, \
-          "from_numpy_matrix() requires numpy: http://scipy.org/ "
-
-
-    G=_prep_create_using(create_using)
-
-    n,m=A.shape
-
-    if n!=m:
-        raise nx.NetworkXError, \
-              "Adjacency matrix is not square. n,m=%s"%(A.shape,)
-
-    G.add_nodes_from(range(n)) # make sure we get isolated nodes
-
-    # get a list of edges
-    x,y=np.asarray(A).nonzero()         
-    G.add_edges_from( ((u,v,{'weight':A[u,v]}) for (u,v) in zip(x,y)) )
-    return G
+    return M.view(np.recarray)
 
 
 def to_scipy_sparse_matrix(G,nodelist=None,dtype=None):
@@ -725,8 +732,8 @@ def to_scipy_sparse_matrix(G,nodelist=None,dtype=None):
     -----
     The matrix entries are populated using the 'weight' edge attribute. When
     an edge does not have the 'weight' attribute, the value of the entry is 1.
-    For multiple edges, the values of the entries are the sums of the edge
-    attributes for each edge.
+
+    For multiple edges the matrix values are the sums of the edge weights.
 
     When `nodelist` does not contain every node in `G`, the matrix is built 
     from the subgraph of `G` that is induced by the nodes in `nodelist`.
@@ -751,8 +758,8 @@ def to_scipy_sparse_matrix(G,nodelist=None,dtype=None):
     try:
         from scipy import sparse
     except ImportError:
-        raise ImportError, \
-          "to_scipy_sparse_matrix() requires scipy: http://scipy.org/ "
+        raise ImportError(\
+          "to_scipy_sparse_matrix() requires scipy: http://scipy.org/ ")
 
     if nodelist is None:
         nodelist = G.nodes()
@@ -801,13 +808,24 @@ def from_scipy_sparse_matrix(A,create_using=None):
     n,m=AA.shape
 
     if n!=m:
-        raise nx.NetworkXError, \
-              "Adjacency matrix is not square. n,m=%s"%(A.shape,)
-
-
+        raise nx.NetworkXError(\
+              "Adjacency matrix is not square. nx,ny=%s"%(A.shape,))
     G.add_nodes_from(range(n)) # make sure we get isolated nodes
 
     for i,row in enumerate(AA.rows):
         for pos,j in enumerate(row):
             G.add_edge(i,j,**{'weight':AA.data[i][pos]})
     return G
+
+# fixture for nose tests
+def setup_module(module):
+    from nose import SkipTest
+    try:
+        import numpy
+    except:
+        raise SkipTest("NumPy not available")
+    try:
+        import scipy
+    except:
+        raise SkipTest("SciPy not available")
+
