@@ -26,7 +26,6 @@ __author__ = """Aric Hagberg (hagberg@lanl.gov)"""
 __all__ = ['write_dot', 'read_dot', 'graphviz_layout', 'pydot_layout',
            'to_pydot', 'from_pydot']
 
-import sys
 from networkx.utils import _get_fh
 import networkx as nx
 
@@ -39,8 +38,7 @@ def write_dot(G,path):
     try:
         import pydot
     except ImportError:
-        raise ImportError, \
-          "write_dot() requires pydot http://dkbza.org/pydot.html/"
+        raise ImportError("write_dot() requires pydot http://dkbza.org/pydot.html/")
     fh=_get_fh(path,'w')
     P=to_pydot(G)
     fh.write(P.to_string())
@@ -48,16 +46,27 @@ def write_dot(G,path):
     return
 
 def read_dot(path):
-    """Return a NetworkX Graph or DiGraph from a dot file on path.
+    """Return a NetworkX MultiGraph or MultiDiGraph from a dot file on path.
 
-    Path can be a string or a file handle.
 
+    Parameters
+    ----------
+    path : filename or file handle
+
+    Returns
+    -------
+    G : NetworkX multigraph
+        A MultiGraph or MultiDiGraph.  
+    
+    Notes
+    -----
+    Use G=nx.Graph(nx.read_dot(path)) to return a Graph instead of a MultiGraph.
     """
     try:
         import pydot
     except ImportError:
-        raise ImportError, \
-          "read_dot() requires pydot http://dkbza.org/pydot.html/"
+        raise ImportError("read_dot() requires pydot",
+                          "http://dkbza.org/pydot.html/")
 
     fh=_get_fh(path,'r')
     data=fh.read()        
@@ -73,11 +82,17 @@ def from_pydot(P):
     P : Pydot graph
       A graph created with Pydot
 
+    Returns
+    -------
+    G : NetworkX multigraph
+        A MultiGraph or MultiDiGraph.  
+    
     Examples
     --------
     >>> K5=nx.complete_graph(5)
     >>> A=nx.to_pydot(K5)
-    >>> G=nx.from_pydot(A)
+    >>> G=nx.from_pydot(A) # return MultiGraph
+    >>> G=nx.Graph(nx.from_pydot(A)) # make a Graph instead of MultiGraph
 
     """
 
@@ -152,8 +167,7 @@ def to_pydot(N, strict=True):
     try:
         import pydot
     except ImportError:
-        raise ImportError, \
-          "to_pydot() requires pydot http://dkbza.org/pydot.html/"
+        raise ImportError("to_pydot() requires pydot http://dkbza.org/pydot.html/")
 
     # set Graphviz graph type
     if N.is_directed():
@@ -165,19 +179,19 @@ def to_pydot(N, strict=True):
     P = pydot.Dot(graph_type=graph_type,strict=strict)
 
     for n,nodedata in N.nodes_iter(data=True):
-        str_nodedata=dict((k,str(v)) for k,v in nodedata.iteritems())
+        str_nodedata=dict((k,str(v)) for k,v in nodedata.items())
         p=pydot.Node(str(n),**str_nodedata)
         P.add_node(p)
 
     if N.is_multigraph():
         for u,v,key,edgedata in N.edges_iter(data=True,keys=True):
-            str_edgedata=dict((k,str(v)) for k,v in edgedata.iteritems())
+            str_edgedata=dict((k,str(v)) for k,v in edgedata.items())
             edge=pydot.Edge(str(u),str(v),key=str(key),**str_edgedata)
             P.add_edge(edge)
         
     else:
         for u,v,edgedata in N.edges_iter(data=True):
-            str_edgedata=dict((k,str(v)) for k,v in edgedata.iteritems())
+            str_edgedata=dict((k,str(v)) for k,v in edgedata.items())
             edge=pydot.Edge(str(u),str(v),**str_edgedata)
             P.add_edge(edge)
 
@@ -198,17 +212,17 @@ def to_pydot(N, strict=True):
 
 
 def pydot_from_networkx(N):
-	"""Create a Pydot graph from a NetworkX graph."""
-        from warnings import warn
-        warn('pydot_from_networkx is replaced by to_pydot', DeprecationWarning)
-        return to_pydot(N)
+    """Create a Pydot graph from a NetworkX graph."""
+    from warnings import warn
+    warn('pydot_from_networkx is replaced by to_pydot', DeprecationWarning)
+    return to_pydot(N)
 
 def networkx_from_pydot(D, create_using=None):
-	"""Create a NetworkX graph from a Pydot graph."""
-        from warnings import warn
-        warn('networkx_from_pydot is replaced by from_pydot', 
-             DeprecationWarning)
-        return from_pydot(D)
+    """Create a NetworkX graph from a Pydot graph."""
+    from warnings import warn
+    warn('networkx_from_pydot is replaced by from_pydot', 
+         DeprecationWarning)
+    return from_pydot(D)
 
 def graphviz_layout(G,prog='neato',root=None, **kwds):
     """Create node positions using Pydot and Graphviz.
@@ -244,8 +258,7 @@ def pydot_layout(G,prog='neato',root=None, **kwds):
     try:
         import pydot
     except ImportError:
-        raise ImportError, \
-          "pydot_layout() requires pydot http://dkbza.org/pydot.html/"
+        raise ImportError("pydot_layout() requires pydot http://dkbza.org/pydot.html/")
 
     P=to_pydot(G)
     if root is not None :
@@ -254,22 +267,31 @@ def pydot_layout(G,prog='neato',root=None, **kwds):
     D=P.create_dot(prog=prog)
 
     if D=="":  # no data returned
-        print "Graphviz layout with %s failed"%(prog)
-        print
-        print "To debug what happened try:"
-        print "P=pydot_from_networkx(G)"
-        print "P.write_dot(\"file.dot\")"
-        print "And then run %s on file.dot"%(prog)
+        print("Graphviz layout with %s failed"%(prog))
+        print()
+        print("To debug what happened try:")
+        print("P=pydot_from_networkx(G)")
+        print("P.write_dot(\"file.dot\")")
+        print("And then run %s on file.dot"%(prog))
         return
 
     Q=pydot.graph_from_dot_data(D)
 
     node_pos={}
     for n in G.nodes():
-        node=Q.get_node(pydot.Node(n).get_name())
+        node=Q.get_node(pydot.Node(str(n)).get_name())
+        if isinstance(node,list):
+            node=node[0]
         pos=node.get_pos()[1:-1] # strip leading and trailing double quotes
         if pos != None:
             xx,yy=pos.split(",")
             node_pos[n]=(float(xx),float(yy))
     return node_pos
 
+# fixture for nose tests
+def setup_module(module):
+    from nose import SkipTest
+    try:
+        import pydot
+    except:
+        raise SkipTest("pydot not available")
