@@ -116,9 +116,9 @@ def draw(G, pos=None, ax=None, hold=None, **kwds):
     try:
         import matplotlib.pylab as pylab
     except ImportError:
-        raise ImportError, "Matplotlib required for draw()"
+        raise ImportError("Matplotlib required for draw()")
     except RuntimeError:
-        print "Matplotlib unable to open display"
+        print("Matplotlib unable to open display")
         raise
 
     cf=pylab.gcf()
@@ -145,7 +145,7 @@ def draw(G, pos=None, ax=None, hold=None, **kwds):
     return
 
 
-def draw_networkx(G, pos=None, ax=None, with_labels=True, **kwds):
+def draw_networkx(G, pos=None, with_labels=True, **kwds):
     """Draw the graph G using Matplotlib.
 
     Draw the graph with Matplotlib with options for node positions,
@@ -231,12 +231,6 @@ def draw_networkx(G, pos=None, ax=None, with_labels=True, **kwds):
     font_family: string
        Font family (default='sans-serif')
 
-    Notes
-    -----
-    Any keywords not listed above are passed through to draw_networkx_nodes(),
-    draw_networkx_edges(), and draw_networkx_labels().  For finer control
-    of drawing you can call those functions directly.
-
     Examples
     --------
     >>> G=nx.dodecahedral_graph()
@@ -261,9 +255,9 @@ def draw_networkx(G, pos=None, ax=None, with_labels=True, **kwds):
     try:
         import matplotlib.pylab as pylab
     except ImportError:
-        raise ImportError, "Matplotlib required for draw()"
+        raise ImportError("Matplotlib required for draw()")
     except RuntimeError:
-        print "Matplotlib unable to open display"
+        print("Matplotlib unable to open display")
         raise
 
     if pos is None:
@@ -338,11 +332,6 @@ def draw_networkx_nodes(G, pos,
        Line width of edges (default =1.0)
 
 
-    Notes
-    -----
-    Any keywords not listed above are passed through to Matplotlib's
-    scatter function.
-
     Examples
     --------
     >>> G=nx.dodecahedral_graph()
@@ -366,9 +355,9 @@ def draw_networkx_nodes(G, pos,
         import matplotlib.pylab as pylab
         import numpy
     except ImportError:
-        raise ImportError, "Matplotlib required for draw()"
+        raise ImportError("Matplotlib required for draw()")
     except RuntimeError:
-        print "Matplotlib unable to open display"
+        print("Matplotlib unable to open display")
         raise
 
 
@@ -383,7 +372,7 @@ def draw_networkx_nodes(G, pos,
 
     try:
         xy=numpy.asarray([pos[v] for v in nodelist])
-    except KeyError,e:
+    except KeyError as e:
         raise nx.NetworkXError('Node %s has no position.'%e)
     except ValueError:
         raise nx.NetworkXError('Bad value in node positions.')
@@ -418,7 +407,7 @@ def draw_networkx_edges(G, pos,
                         ax=None,
                         arrows=True,
                         **kwds):
-    """Draw the edges of the graph G
+    """Draw the edges of the graph G.
 
     This draws only the edges of the graph G.
 
@@ -488,9 +477,9 @@ def draw_networkx_edges(G, pos,
         from matplotlib.collections import LineCollection
         import numpy
     except ImportError:
-        raise ImportError, "Matplotlib required for draw()"
+        raise ImportError("Matplotlib required for draw()")
     except RuntimeError:
-        print "Matplotlib unable to open display"
+        print("Matplotlib unable to open display")
         raise
 
     if ax is None:
@@ -564,7 +553,7 @@ def draw_networkx_edges(G, pos,
         mpl_version=matplotlib.__version__[0:-3]
     elif mpl_version.endswith('pre'):
         mpl_version=matplotlib.__version__[0:-3]
-    if map(int,mpl_version.split('.'))>=[0,87,7]:
+    if list(map(int,mpl_version.split('.')))>=[0,87,7]:
         if edge_colors is None:
             if edge_cmap is not None: assert(isinstance(edge_cmap, Colormap))
             edge_collection.set_array(numpy.asarray(edge_color))
@@ -653,7 +642,7 @@ def draw_networkx_labels(G, pos,
                          alpha=1.0,
                          ax=None,
                          **kwds):
-    """Draw node labels on the graph G
+    """Draw node labels on the graph G.
 
     Parameters
     ----------
@@ -708,19 +697,23 @@ def draw_networkx_labels(G, pos,
         import matplotlib.pylab as pylab
         import matplotlib.cbook as cb
     except ImportError:
-        raise ImportError, "Matplotlib required for draw()"
+        raise ImportError("Matplotlib required for draw()")
     except RuntimeError:
-        print "Matplotlib unable to open display"
+        print("Matplotlib unable to open display")
         raise
 
     if ax is None:
         ax=pylab.gca()
 
     if labels is None:
-        labels=dict(zip(G.nodes(),G.nodes()))
+        labels=dict(list(zip(G.nodes(),G.nodes())))
+
+    # set optional alignment
+    horizontalalignment=kwds.get('horizontalalignment','center')
+    verticalalignment=kwds.get('verticalalignment','center')
 
     text_items={}  # there is no text collection so we'll fake one        
-    for (n,label) in labels.items():
+    for n, label in labels.items():
         (x,y)=pos[n]
         if not cb.is_string_like(label):
             label=str(label) # this will cause "1" and 1 to be labeled the same
@@ -730,8 +723,8 @@ def draw_networkx_labels(G, pos,
                   color=font_color,
                   family=font_family,
                   weight=font_weight,
-                  horizontalalignment='center',
-                  verticalalignment='center',
+                  horizontalalignment=horizontalalignment,
+                  verticalalignment=verticalalignment,
                   transform = ax.transData,
                   clip_on=True,
                   )
@@ -748,6 +741,7 @@ def draw_networkx_edge_labels(G, pos,
                               alpha=1.0,
                               bbox=None,
                               ax=None,
+                              rotate=True,
                               **kwds):
     """Draw edge labels.
 
@@ -812,32 +806,35 @@ def draw_networkx_edge_labels(G, pos,
         import matplotlib.cbook as cb
         import numpy
     except ImportError:
-        raise ImportError, "Matplotlib required for draw()"
+        raise ImportError("Matplotlib required for draw()")
     except RuntimeError:
-        print "Matplotlib unable to open display"
+        print("Matplotlib unable to open display")
         raise
 
     if ax is None:
         ax=pylab.gca()
     if edge_labels is None:
-        labels=dict(zip(G.edges(),[d for u,v,d in G.edges(data=True)]))
+        labels=dict(list(zip(G.edges(),[d for u,v,d in G.edges(data=True)])))
     else:
         labels = edge_labels
     text_items={} 
-    for ((n1,n2),label) in labels.items():
+    for (n1,n2), label in labels.items():
         (x1,y1)=pos[n1]
         (x2,y2)=pos[n2]
-        (x,y) = ((x1+x2)/2, (y1+y2)/2)
-        angle=numpy.arctan2(y2-y1,x2-x1)/(2.0*numpy.pi)*360 # degrees
-        # make label orientation "right-side-up"
-        if angle > 90: 
-            angle-=180
-        if angle < - 90: 
-            angle+=180
-        # transform data coordinate angle to screen coordinate angle
-        xy=numpy.array((x,y))
-        trans_angle=ax.transData.transform_angles(numpy.array((angle,)),
-                                                  xy.reshape((1,2)))[0]
+        (x,y) = (float(x1+x2)/2.0, float(y1+y2)/2.0)
+        if rotate: 
+            angle=numpy.arctan2(y2-y1,x2-x1)/(2.0*numpy.pi)*360 # degrees
+            # make label orientation "right-side-up"
+            if angle > 90: 
+                angle-=180
+            if angle < - 90: 
+                angle+=180
+            # transform data coordinate angle to screen coordinate angle
+            xy=numpy.array((x,y))
+            trans_angle=ax.transData.transform_angles(numpy.array((angle,)),
+                                                      xy.reshape((1,2)))[0]
+        else:
+            trans_angle=0.0
         # use default box of white with white border
         if bbox is None:
             bbox = dict(boxstyle='round',
@@ -846,14 +843,19 @@ def draw_networkx_edge_labels(G, pos,
                         )
         if not cb.is_string_like(label):
             label=str(label) # this will cause "1" and 1 to be labeled the same
+
+        # set optional alignment
+        horizontalalignment=kwds.get('horizontalalignment','center')
+        verticalalignment=kwds.get('verticalalignment','center')
+
         t=ax.text(x, y,
                   label,
                   size=font_size,
                   color=font_color,
                   family=font_family,
                   weight=font_weight,
-                  horizontalalignment='center',
-                  verticalalignment='center',
+                  horizontalalignment=horizontalalignment,
+                  verticalalignment=verticalalignment,
                   rotation=trans_angle,
                   transform = ax.transData,
                   bbox = bbox,
@@ -867,7 +869,7 @@ def draw_networkx_edge_labels(G, pos,
 
 
 def draw_circular(G, **kwargs):
-    """Draw the graph G with a circular layout"""
+    """Draw the graph G with a circular layout."""
     draw(G,circular_layout(G),**kwargs)
     
 def draw_random(G, **kwargs):
@@ -879,21 +881,31 @@ def draw_spectral(G, **kwargs):
     draw(G,spectral_layout(G),**kwargs)
 
 def draw_spring(G, **kwargs):
-    """Draw the graph G with a spring layout"""
+    """Draw the graph G with a spring layout."""
     draw(G,spring_layout(G),**kwargs)
 
 def draw_shell(G, **kwargs):
-    """Draw networkx graph with shell layout"""
+    """Draw networkx graph with shell layout."""
     nlist = kwargs.get('nlist', None)
     if nlist != None:        
         del(kwargs['nlist'])
     draw(G,shell_layout(G,nlist=nlist),**kwargs)
 
 def draw_graphviz(G, prog="neato", **kwargs):
-    """Draw networkx graph with graphviz layout"""
+    """Draw networkx graph with graphviz layout."""
     pos=nx.drawing.graphviz_layout(G,prog)
     draw(G,pos,**kwargs)
 
 def draw_nx(G,pos,**kwds):
-    """For backward compatibility; use draw or draw_networkx"""
+    """For backward compatibility; use draw or draw_networkx."""
     draw(G,pos,**kwds)
+
+# fixture for nose tests
+def setup_module(module):
+    from nose import SkipTest
+    try:
+        import matplotlib as mpl
+        mpl.use('PS',warn=False)
+        import pylab
+    except:
+        raise SkipTest("matplotlib not available")
