@@ -3,7 +3,7 @@
 Generators for random graphs.
 
 """
-#    Copyright (C) 2004-2009 by 
+#    Copyright (C) 2004-2011 by 
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
@@ -18,50 +18,10 @@ import math
 import networkx as nx
 from networkx.generators.classic import empty_graph, path_graph, complete_graph
 
-# until we require python2.5 fall back to pure Python defaultdict 
-# from http://code.activestate.com/recipes/523034/
-try:
-    from collections import defaultdict
-except:
-    class defaultdict(dict):
-        def __init__(self, default_factory=None, *a, **kw):
-            if (default_factory is not None and
-                not hasattr(default_factory, '__call__')):
-                raise TypeError('first argument must be callable')
-            dict.__init__(self, *a, **kw)
-            self.default_factory = default_factory
-        def __getitem__(self, key):
-            try:
-                return dict.__getitem__(self, key)
-            except KeyError:
-                return self.__missing__(key)
-        def __missing__(self, key):
-            if self.default_factory is None:
-                raise KeyError(key)
-            self[key] = value = self.default_factory()
-            return value
-        def __reduce__(self):
-            if self.default_factory is None:
-                args = tuple()
-            else:
-                args = self.default_factory,
-            return type(self), args, None, None, self.items()
-        def copy(self):
-            return self.__copy__()
-        def __copy__(self):
-            return type(self)(self.default_factory, self)
-        def __deepcopy__(self, memo):
-            import copy
-            return type(self)(self.default_factory,
-                              copy.deepcopy(self.items()))
-        def __repr__(self):
-            return 'defaultdict(%s, %s)' % (self.default_factory,
-                                            dict.__repr__(self))
-
+from collections import defaultdict
 
 __all__ = ['fast_gnp_random_graph',
            'gnp_random_graph',
-           'directed_gnp_random_graph',
            'dense_gnm_random_graph',
            'gnm_random_graph',
            'erdos_renyi_graph',
@@ -83,13 +43,8 @@ __all__ = ['fast_gnp_random_graph',
 #-------------------------------------------------------------------------
 
 
-def fast_gnp_random_graph(n, p, create_using=None, seed=None):
-    """Return a random graph G_{n,p}.
-
-    The G_{n,p} graph choses each of the possible [n(n-1)]/2 edges
-    with probability p.
-
-    Sometimes called Erdős-Rényi graph, or binomial graph.
+def fast_gnp_random_graph(n, p, seed=None, directed=False):
+    """Return a random graph G_{n,p} (Erdős-Rényi graph, binomial graph).
 
     Parameters
     ----------
@@ -97,55 +52,73 @@ def fast_gnp_random_graph(n, p, create_using=None, seed=None):
         The number of nodes.
     p : float
         Probability for edge creation.
-    create_using :  graph, optional (default Graph)
-        Use specified graph as a container.
     seed : int, optional
         Seed for random number generator (default=None). 
+    directed : bool, optional (default=False)
+        If True return a directed graph 
       
     Notes
     -----
+    The G_{n,p} graph algorithm chooses each of the [n(n-1)]/2
+    (undirected) or n(n-1) (directed) possible edges with probability p.
+
     This algorithm is O(n+m) where m is the expected number of
     edges m=p*n*(n-1)/2.
     
-    It should be faster than gnp_random_graph when p is small, and
-    the expected number of edges is small, (sparse graph).
+    It should be faster than gnp_random_graph when p is small and
+    the expected number of edges is small (sparse graph).
+
+    See Also
+    --------
+    gnp_random_graph
 
     References
     ----------
-    .. [1] Batagelj and Brandes,
+    .. [1] Vladimir Batagelj and Ulrik Brandes, 
        "Efficient generation of large random networks",
        Phys. Rev. E, 71, 036113, 2005.
     """
-    if create_using is not None and create_using.is_directed():
-        raise nx.NetworkXError("Directed Graph not supported")
-    G=empty_graph(n,create_using)
+    G = empty_graph(n)
     G.name="fast_gnp_random_graph(%s,%s)"%(n,p)
 
     if not seed is None:
         random.seed(seed)
 
-    v=1  # Nodes in graph are from 0,n-1 (this is the second node index).
-    w=-1
-    lp=math.log(1.0-p)  
+    if p <= 0 or p >= 1:
+        return nx.gnp_random_graph(n,p)
 
-    while v<n:
-        lr=math.log(1.0-random.random())
-        w=w+1+int(lr/lp)
-        while w>=v and v<n:
-            w=w-v
-            v=v+1
-        if v<n:
-            G.add_edge(v,w)
+    v = 1  # Nodes in graph are from 0,n-1 (this is the second node index).
+    w = -1
+    lp = math.log(1.0 - p)  
+
+    if directed:
+        G=nx.DiGraph(G)
+        while v < n:
+            lr = math.log(1.0 - random.random())
+            w = w + 1 + int(lr/lp)
+            while  w>= n and v < n:
+                w = w - n
+                v = v + 1
+            if v < n:
+                G.add_edge(v, w)
+    else:
+        while v < n:
+            lr = math.log(1.0 - random.random())
+            w = w + 1 + int(lr/lp)
+            while w >= v and v < n:
+                w = w - v
+                v = v + 1
+            if v < n:
+                G.add_edge(v, w)
     return G
 
 
-def gnp_random_graph(n, p, create_using=None, seed=None):
-    """Return a random graph G_{n,p}.
+def gnp_random_graph(n, p, seed=None, directed=False):
+    """Return a random graph G_{n,p} (Erdős-Rényi graph, binomial graph).
 
-    Choses each of the possible [n(n-1)]/2 edges with probability p.
-    This is the same as binomial_graph and erdos_renyi_graph. 
+    Chooses each of the possible edges with probability p.
 
-    Sometimes called Erdős-Rényi graph, or binomial graph.
+    This is also called binomial_graph and erdos_renyi_graph. 
 
     Parameters
     ----------
@@ -153,38 +126,47 @@ def gnp_random_graph(n, p, create_using=None, seed=None):
         The number of nodes.
     p : float
         Probability for edge creation.
-    create_using :  graph, optional (default Graph)
-        Use specified graph as a container.
     seed : int, optional
         Seed for random number generator (default=None). 
-      
-    See Also
+    directed : bool, optional (default=False)
+        If True return a directed graph 
+    
+    See Also  
     --------
-    fast_gnp_random_graph()
+    fast_gnp_random_graph
 
     Notes
     -----
     This is an O(n^2) algorithm.  For sparse graphs (small p) see
-    fast_gnp_random_graph. 
+    fast_gnp_random_graph for a faster algorithm.
 
     References
     ----------
     .. [1] P. Erdős and A. Rényi, On Random Graphs, Publ. Math. 6, 290 (1959).
     .. [2] E. N. Gilbert, Random Graphs, Ann. Math. Stat., 30, 1141 (1959).
-
     """
-    if create_using is not None and create_using.is_directed():
-        raise nx.NetworkXError("Directed Graph not supported")
-    G=empty_graph(n,create_using)
+    if directed:
+        G=nx.DiGraph()
+    else:
+        G=nx.Graph()
+    G.add_nodes_from(range(n))
     G.name="gnp_random_graph(%s,%s)"%(n,p)
+    if p<=0:
+        return G
+    if p>=1:
+        return complete_graph(n,create_using=G)
 
     if not seed is None:
         random.seed(seed)
 
-    for u in xrange(n):
-        for v in xrange(u+1,n):
-            if random.random() < p:
-                G.add_edge(u,v)
+    if G.is_directed():
+        edges=itertools.permutations(range(n),2)
+    else:
+        edges=itertools.combinations(range(n),2)
+
+    for e in edges:
+        if random.random() < p:        
+            G.add_edge(*e)
     return G
 
 
@@ -192,55 +174,7 @@ def gnp_random_graph(n, p, create_using=None, seed=None):
 binomial_graph=gnp_random_graph
 erdos_renyi_graph=gnp_random_graph
 
-def directed_gnp_random_graph(n, p, create_using=None, seed=None):
-    """Return a directed random graph.
-
-    Chooses each of the possible n(n-1) edges with probability p.
-
-    This is a directed version of G_np.
-
-    Parameters
-    ----------
-    n : int
-        The number of nodes.
-    p : float
-        Probability for edge creation.
-    create_using :  graph, optional (default DiGraph)
-        Use specified graph as a container.
-    seed : int, optional
-        Seed for random number generator (default=None). 
-      
-    See Also
-    --------
-    gnp_random_graph()
-    fast_gnp_random_graph()
-
-    Notes
-    -----
-    This is an O(n^2) algorithm.  
-
-    References
-    ----------
-    .. [1] P. Erdős and A. Rényi, On Random Graphs, Publ. Math. 6, 290 (1959).
-    .. [2] E. N. Gilbert, Random Graphs, Ann. Math. Stat., 30, 1141 (1959).
-
-    """
-    if create_using is None:
-        create_using=nx.DiGraph()
-    G=empty_graph(n,create_using)
-    G.name="directed gnp_random_graph(%s,%s)"%(n,p)
-
-    if not seed is None:
-        random.seed(seed)
-
-    for u in xrange(n):
-        for v in xrange(n):
-            if u==v: continue
-            if random.random() < p:
-                G.add_edge(u,v)
-    return G
-
-def dense_gnm_random_graph(n, m, create_using=None, seed=None):
+def dense_gnm_random_graph(n, m, seed=None):
     """Return the random graph G_{n,m}.
 
     Gives a graph picked randomly out of the set of all graphs
@@ -253,8 +187,6 @@ def dense_gnm_random_graph(n, m, create_using=None, seed=None):
         The number of nodes.
     m : int
         The number of edges.
-    create_using :  graph, optional (default Graph)
-        Use specified graph as a container.
     seed : int, optional
         Seed for random number generator (default=None). 
       
@@ -266,25 +198,19 @@ def dense_gnm_random_graph(n, m, create_using=None, seed=None):
     -----
     Algorithm by Keith M. Briggs Mar 31, 2006.
     Inspired by Knuth's Algorithm S (Selection sampling technique),
-    in section 3.4.2 of
+    in section 3.4.2 of [1]_.
 
     References
     ----------
-    .. [1] Donald E. Knuth,
-        The Art of Computer Programming,
-        Volume 2 / Seminumerical algorithms
-        Third Edition, Addison-Wesley, 1997.
- 
+    .. [1] Donald E. Knuth, The Art of Computer Programming,
+        Volume 2/Seminumerical algorithms, Third Edition, Addison-Wesley, 1997.
     """
     mmax=n*(n-1)/2
     if m>=mmax:
-        G=complete_graph(n,create_using)
+        G=complete_graph(n)
     else:
-        if create_using is not None and create_using.is_directed():
-            raise nx.NetworkXError("Directed Graph not supported")
-        G=empty_graph(n,create_using)
-
-        G.name="dense_gnm_random_graph(%s,%s)"%(n,m)
+        G=empty_graph(n)
+    G.name="dense_gnm_random_graph(%s,%s)"%(n,m)
   
     if n==1 or m>=mmax:
         return G
@@ -307,10 +233,10 @@ def dense_gnm_random_graph(n, m, create_using=None, seed=None):
             u+=1
             v=u+1
 
-def gnm_random_graph(n, m, create_using=None, seed=None):
+def gnm_random_graph(n, m, seed=None, directed=False):
     """Return the random graph G_{n,m}.
 
-    Gives a graph picked randomly out of the set of all graphs
+    Produces a graph picked randomly out of the set of all graphs
     with n nodes and m edges.
 
     Parameters
@@ -319,15 +245,16 @@ def gnm_random_graph(n, m, create_using=None, seed=None):
         The number of nodes.
     m : int
         The number of edges.
-    create_using :  graph, optional (default Graph)
-        Use specified graph as a container.
     seed : int, optional
         Seed for random number generator (default=None). 
-      
+    directed : bool, optional (default=False)
+        If True return a directed graph 
     """
-    if create_using is not None and create_using.is_directed():
-        raise nx.NetworkXError("Directed Graph not supported")
-    G=empty_graph(n,create_using)
+    if directed:
+        G=nx.DiGraph()
+    else:
+        G=nx.Graph()
+    G.add_nodes_from(range(n))
     G.name="gnm_random_graph(%s,%s)"%(n,m)
 
     if seed is not None:
@@ -335,9 +262,11 @@ def gnm_random_graph(n, m, create_using=None, seed=None):
 
     if n==1:
         return G
-
-    if m>=n*(n-1)/2:
-        return complete_graph(n,create_using)
+    max_edges=n*(n-1)
+    if not directed:
+        max_edges/=2.0
+    if m>=max_edges:
+        return complete_graph(n,create_using=G)
 
     nlist=G.nodes()
     edge_count=0
@@ -347,15 +276,12 @@ def gnm_random_graph(n, m, create_using=None, seed=None):
         v = random.choice(nlist)
         if u==v or G.has_edge(u,v):
             continue
-        # is this faster?
-        # (u,v)=random.sample(nlist,2)
-        #  if G.has_edge(u,v):
-        #      continue
         else:
             G.add_edge(u,v)
             edge_count=edge_count+1
     return G
 
+
 def newman_watts_strogatz_graph(n, k, p, create_using=None, seed=None):
     """Return a Newman-Watts-Strogatz small world graph.
 
@@ -394,6 +320,8 @@ def newman_watts_strogatz_graph(n, k, p, create_using=None, seed=None):
     """
     if seed is not None:
         random.seed(seed)
+    if k>=n // 2: 
+        raise nx.NetworkXError("k>=n/2, choose smaller k or larger n")
     if create_using is not None and create_using.is_directed():
         raise nx.NetworkXError("Directed Graph not supported")
     G=empty_graph(n,create_using)
@@ -401,7 +329,7 @@ def newman_watts_strogatz_graph(n, k, p, create_using=None, seed=None):
     nlist = G.nodes()
     fromv = nlist
     # connect the k/2 neighbors
-    for n in range(1, k/2+1):
+    for n in range(1, k // 2+1):
         tov = fromv[n:] + fromv[0:n] # the first n are now last
         for i in range(len(fromv)):
             G.add_edge(fromv[i], tov[i])
@@ -460,6 +388,8 @@ def watts_strogatz_graph(n, k, p, create_using=None, seed=None):
        Collective dynamics of small-world networks,
        Nature, 393, pp. 440--442, 1998.
     """
+    if k>=n/2: 
+        raise nx.NetworkXError("k>=n/2, choose smaller k or larger n")
     if create_using is None:
         G = nx.Graph()
     elif create_using.is_directed():
@@ -472,15 +402,15 @@ def watts_strogatz_graph(n, k, p, create_using=None, seed=None):
         random.seed(seed)
 
     G.name="watts_strogatz_graph(%s,%s,%s)"%(n,k,p)
-    nodes = range(n) # nodes are labeled 0 to n-1
+    nodes = list(range(n)) # nodes are labeled 0 to n-1
     # connect each node to k/2 neighbors
-    for j in range(1, k/2+1):
+    for j in range(1, k // 2+1):
         targets = nodes[j:] + nodes[0:j] # first j nodes are now last in list
         G.add_edges_from(zip(nodes,targets))
     # rewire edges from each node
     # loop over all nodes in order (label) and neighbors in order (distance)
     # no self loops or multiple edges allowed
-    for j in range(1, k/2+1): # outer loop is neighbors
+    for j in range(1, k // 2+1): # outer loop is neighbors
         targets = nodes[j:] + nodes[0:j] # first j nodes are now last in list
         # inner loop in node order
         for u,v in zip(nodes,targets): 
@@ -521,10 +451,9 @@ def connected_watts_strogatz_graph(n, k, p, tries=100, create_using=None, seed=N
     watts_strogatz_graph()
 
     """
-    from nx.algorithms.traversal.component import is_connected
     G = watts_strogatz_graph(n, k, p, create_using, seed)
     t=1
-    while not is_connected(G):
+    while not nx.is_connected(G):
         G = watts_strogatz_graph(n, k, p, create_using, seed)
         t=t+1
         if t>tries:
@@ -568,7 +497,7 @@ def random_regular_graph(d, n, create_using=None, seed=None):
        Generating random regular graphs,
        Proceedings of the thirty-fifth ACM symposium on Theory of computing,
        San Diego, CA, USA, pp 213--222, 2003.
-       http://doi.acm.org/10.1145/780542.780576
+       http://portal.acm.org/citation.cfm?id=780542.780576
     """
     if (n * d) % 2 != 0:
         raise nx.NetworkXError("n * d must be even")
@@ -610,13 +539,13 @@ def random_regular_graph(d, n, create_using=None, seed=None):
         # Attempt to create an edge set
 
         edges = set()
-        stubs = range(n) * d
+        stubs = list(range(n)) * d
 
         while stubs:
-            potential_edges = defaultdict(itertools.repeat(0).next)
+            potential_edges = defaultdict(lambda: 0)
             random.shuffle(stubs)
             stubiter = iter(stubs)
-            for s1, s2 in itertools.izip(stubiter, stubiter):
+            for s1, s2 in zip(stubiter, stubiter):
                 if s1 > s2:
                     s1, s2 = s2, s1
                 if s1 != s2 and ((s1, s2) not in edges):
@@ -628,8 +557,8 @@ def random_regular_graph(d, n, create_using=None, seed=None):
             if not _suitable(edges, potential_edges):
                 return None # failed to find suitable edge set
 
-            stubs = [node for node, potential in potential_edges.iteritems()
-                     for _ in xrange(potential)]
+            stubs = [node for node, potential in potential_edges.items()
+                     for _ in range(potential)]
         return edges 
 
     # Even though a suitable edge set exists, 
@@ -644,7 +573,18 @@ def random_regular_graph(d, n, create_using=None, seed=None):
 
     return G
 
+def _random_subset(seq,m):
+    """ Return m unique elements from seq.
 
+    This differs from random.sample which can return repeated
+    elements if seq holds repeated elements.
+    """
+    targets=set()
+    while len(targets)<m:
+        x=random.choice(seq)
+        targets.add(x)
+    return targets
+    
 def barabasi_albert_graph(n, m, create_using=None, seed=None):
     """Return random graph using Barabási-Albert preferential attachment model.
         
@@ -691,7 +631,7 @@ def barabasi_albert_graph(n, m, create_using=None, seed=None):
     G=empty_graph(m,create_using)  
     G.name="barabasi_albert_graph(%s,%s)"%(n,m)
     # Target nodes for new edges
-    targets=range(m)  
+    targets=list(range(m))
     # List of existing nodes, with nodes repeated once for each adjacent edge 
     repeated_nodes=[]     
     # Start adding the other n-m nodes. The first node is m.
@@ -705,10 +645,7 @@ def barabasi_albert_graph(n, m, create_using=None, seed=None):
         repeated_nodes.extend([source]*m) 
         # Now choose m unique nodes from the existing nodes 
         # Pick uniformly from repeated_nodes (preferential attachement) 
-        targets=set()
-        while len(targets)<m:
-            x=random.choice(repeated_nodes)
-            targets.add(x)
+        targets = _random_subset(repeated_nodes,m)
         source += 1
     return G
 
@@ -774,7 +711,7 @@ def powerlaw_cluster_graph(n, m, p, create_using=None, seed=None):
                            # with nodes repeated once for each adjacent edge 
     source=m               # next node is m
     while source<n:        # Now add the other n-1 nodes
-        possible_targets=random.sample(repeated_nodes,m)
+        possible_targets = _random_subset(repeated_nodes,m)
         # do one preferential attachment for new node
         target=possible_targets.pop()
         G.add_edge(source,target)  
@@ -833,7 +770,7 @@ def random_lobster(n, p1, p2, create_using=None, seed=None):
     L.name="random_lobster(%d,%s,%s)"%(n,p1,p2)
     # build caterpillar: add edges to path graph with probability p1
     current_node=llen-1
-    for n in xrange(llen):
+    for n in range(llen):
         if random.random()<p1: # add fuzzy caterpillar parts
             current_node+=1
             L.add_edge(n,current_node)
@@ -929,7 +866,7 @@ def random_powerlaw_tree(n, gamma=3, create_using=None, seed=None, tries=100):
     the sequence makes a tree (#edges=#nodes-1).  
 
     """
-    from nx.generators.degree_seq import degree_sequence_tree
+    from networkx.generators.degree_seq import degree_sequence_tree
     try:
         s=random_powerlaw_tree_sequence(n,
                                         gamma=gamma,
