@@ -2,13 +2,16 @@
 import sys
 from os import path
 
-def run(verbosity=1):
+def run(verbosity=1,doctest=False):
     """Run NetworkX tests.
 
     Parameters
     ----------
     verbosity: integer, optional
       Level of detail in test reports.  Higher numbers provide  more detail.  
+
+    doctest: bool, optional
+      True to run doctests in code modules
     """
     try:
         import nose
@@ -21,9 +24,10 @@ def run(verbosity=1):
     nx_install_dir=path.join(path.dirname(__file__), path.pardir)
     argv=[' ','--verbosity=%d'%verbosity,
           '-w',nx_install_dir,
-          '--with-doctest',
-          '--doctest-extension=txt',
-          '--exe']
+          '-exe']
+    if doctest:
+        argv.extend(['--with-doctest','--doctest-extension=txt'])
+
     nose.run(argv=argv)
 
 if __name__=="__main__":
