@@ -7,19 +7,19 @@ Do not add this file to the repository.
 
 import datetime
 
-version = '1.4'
-date = 'Sun Jan 23 15:44:00 2011'
+version = '1.5'
+date = 'Sat Jun  4 09:49:07 2011'
 
 # Was NetworkX built from a development version? If so, remember that the major
 # and minor versions reference the "target" (rather than "current") release.
 dev = False
 
 # Format: (name, major, min, revision)
-version_info = ('networkx', 1, 4, '7f42e7950c95')
+version_info = ('networkx', 1, 5, '7534d8095e2c')
 
 # Format: a 'datetime.datetime' instance
-date_info = datetime.datetime(2011, 1, 23, 15, 44, 0, 82605)
+date_info = datetime.datetime(2011, 6, 4, 9, 49, 7, 997782)
 
 # Format: (vcs, vcs_tuple)
-vcs_info = ('mercurial', ('7f42e7950c95', 'tip'))
+vcs_info = ('mercurial', ('7534d8095e2c', 'networkx-1.5'))
 
